################################################################################################################################
#Menu
################################################################################################################################
import os
import inspect
import shutil
import time

class New:
    def __init__(self, Global, Project):
        try:
            self.Global = Global
            self.Project = Project
            self.Widget = []
            self.Error = False
            
            Fixture = self.Global['GUI'].Locate(60, 50, 25, 25)
            self.Frame = self.Global['Gluonix'].Frame(self.Global['GUI'])
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='white', Border_Size=1, Display=False)
            self.Frame.Create()
            self.Project.Widget.append(self.Frame)
            
                #Info Label
            Fixture = self.Frame.Locate(70, 6, 3, 8)
            self.Info_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Info_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Info_Label.Config(Foreground='black', Font_Size=9, Font_Weight='normal', Align='w', Border_Size=0)
            self.Info_Label.Config(Value="Enter the name and location for the new project")
            self.Info_Label.Create()
            
                #Name Label
            Fixture = self.Frame.Locate(10, 6, 3, 16)
            self.Name_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Name_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Name_Label.Config(Foreground='black', Value="Name:", Font_Size=9, Font_Weight='normal', Align='w', Border_Size=0)
            self.Name_Label.Create()
            
                #Name Entry
            Fixture = self.Frame.Locate(70, 6, 13, 16)
            self.Name_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Name_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Name_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='left', Border_Size=1)
            self.Name_Entry.Bind(On_Key_Release=lambda E: self.Update_Path())
            self.Name_Entry.Create()
            
                #Location Label
            Fixture = self.Frame.Locate(10, 6, 3, 24)
            self.Name_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Name_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Name_Label.Config(Foreground='black', Value="Location:", Font_Size=9, Font_Weight='normal', Align='w', Border_Size=0)
            self.Name_Label.Create()
            
                #Location Entry
            Fixture = self.Frame.Locate(70, 6, 13, 24)
            self.Location_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Location_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Location_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='left', Border_Size=1)
            self.Location_Entry.Bind(On_Key_Release=lambda E: self.Update_Path())
            self.Location_Entry.Create()
            
                #Location Browser
            Fixture = self.Frame.Locate(12, 6, 85, 24)
            self.Location_Browser = self.Global['Gluonix'].Label(self.Frame)
            self.Location_Browser.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Location_Browser.Config(Background='#e1e1e1', Foreground='black', Value='Browser...', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Location_Browser.Bind(On_Hover_In=lambda E: self.Location_Browser.Config(Border_Color='#0078d7', Background='#d5dcf0'))
            self.Location_Browser.Bind(On_Hover_Out=lambda E: self.Location_Browser.Config(Border_Color='#adadad', Background='#e1e1e1'))
            self.Location_Browser.Bind(On_Click=lambda E: self.Project_Folder())
            self.Location_Browser.Create()
            
                #Project Path Label
            Fixture = self.Frame.Locate(50, 4, 13, 34)
            self.Project_Path_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Project_Path_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Project_Path_Label.Config(Foreground='black', Font_Size=9, Font_Weight='normal', Align='w', Border_Size=0)
            self.Project_Path_Label.Config(Value="The project will be created in")
            self.Project_Path_Label.Create()
            self.Location_Browser.Create()
            
                #Project Path
            Fixture = self.Frame.Locate(70, 15, 13, 38.5)
            self.Project_Path = self.Global['Gluonix'].Label(self.Frame)
            self.Project_Path.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Project_Path.Config(Foreground='black', Font_Size=9, Font_Weight='normal', Align='nw', Border_Size=0)
            self.Project_Path.Create()
            
                #Ok Button
            Fixture = self.Frame.Locate(12, 6, 70, 88)
            self.Ok_Button = self.Global['Gluonix'].Label(self.Frame)
            self.Ok_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Ok_Button.Config(Background='#e1e1e1', Foreground='black', Value='OK', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Ok_Button.Bind(On_Hover_In=lambda E: self.Ok_Button.Config(Border_Color='#0078d7', Background='#d5dcf0'))
            self.Ok_Button.Bind(On_Hover_Out=lambda E: self.Ok_Button.Config(Border_Color='#adadad', Background='#e1e1e1'))
            self.Ok_Button.Bind(On_Click=lambda E: self.Create())
            self.Ok_Button.Create()
            
                #Cancel Button
            Fixture = self.Frame.Locate(12, 6, 85, 88)
            self.Cancel_Button = self.Global['Gluonix'].Label(self.Frame)
            self.Cancel_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Cancel_Button.Config(Background='#e1e1e1', Foreground='black', Value='Cancel', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Cancel_Button.Bind(On_Hover_In=lambda E: self.Cancel_Button.Config(Border_Color='#0078d7', Background='#d5dcf0'))
            self.Cancel_Button.Bind(On_Hover_Out=lambda E: self.Cancel_Button.Config(Border_Color='#adadad', Background='#e1e1e1'))
            self.Cancel_Button.Bind(On_Click=lambda E: self.Frame.Hide())
            self.Cancel_Button.Create()
                
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Reset(self):
        try:
            self.Name_Entry.Set("Project1")
            self.Location_Entry.Set(os.path.join(os.path.expanduser('~'), 'Documents'))
            self.Project_Path.Set(self.Location_Entry.Get()+"/"+self.Name_Entry.Get())
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Project_Folder(self):
        try:
            Folder_Path = self.Global['GUI'].Folder(Initial=self.Location_Entry.Get())
            if Folder_Path:
                self.Location_Entry.Set(Folder_Path)
                self.Update_Path()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
            
    def Update_Path(self):
        try:
            Error = False
            if os.path.exists(self.Location_Entry.Get()):
                self.Location_Entry.Config(Border_Color='black')
            else:
                self.Location_Entry.Config(Border_Color='red')
                Error = True
            if self.Name_Entry.Get():
                self.Name_Entry.Config(Border_Color='black')
            else:
                self.Name_Entry.Config(Border_Color='red')
                Error = True
            if Error:
                self.Error = True
                self.Project_Path.Set('Project path does not exists!')
            else:
                self.Error = False
                self.Project_Path.Set(self.Location_Entry.Get()+"/"+self.Name_Entry.Get())
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Create(self):
        try:
            Project_Path = self.Location_Entry.Get()
            Project_Name = self.Name_Entry.Get()
            if not self.Error:
                if self.Global['Custom'].Is_Empty(Project_Path):
                    self.Global['Message'].Hide()
                    with open(f'{Project_Path}/{Project_Name}.ng', 'w') as File:
                        File.write(Project_Name)
                    os.makedirs(f'{Project_Path}/Data')
                    os.makedirs(f'{Project_Path}/Data/File')
                    Icon = self.Global['Custom'].MD5(Project_Name+str(time.time()))
                    shutil.copy(self.Global['Image']("Icon", "ico"), f'{Project_Path}/Data/File/{Icon}')
                    shutil.copy(self.Global['Data']('NGD.dll'), f'{Project_Path}/Data/NGD.dll')
                    TempDatabase = self.Global['Gluonix'].SQL(f'{Project_Path}/Data/NGD.dll')
                    TempDatabase.Post(f"UPDATE `Variable` SET `DATA`='{Project_Name}' WHERE `ID`='Name'")
                    TempDatabase.Post(f"UPDATE `Variable` SET `DATA`='{Project_Name}' WHERE `ID`='Title'")
                    TempDatabase.Post(f"UPDATE `Variable` SET `DATA`='{Icon}' WHERE `ID`='Icon'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Title`='{Project_Name}' WHERE `ID`='Root'")
                    TempDatabase.Close()
                    self.Project.Home.Panel.Overview.Project_Path = Project_Path
                    self.Project.Home.Panel.Overview.Update()
                    self.Frame.Hide()
                else:
                    self.Global['Message'].Show('Error', 'Project Folder Is Not Empty')
            else:
                if not os.path.exists(Project_Path):
                    self.Global['Message'].Show('Error', 'Path Does Not Exist')
                if not Project_Name:
                    self.Global['Message'].Show('Error', 'Enter A Project Name')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


################################################################################################################################
#Configure_Entry
################################################################################################################################
import inspect
from tkinter import colorchooser

class Configure_Entry:
    def __init__(self, Global, Configure):
        try:
            self.Global = Global
            self.Configure = Configure
            self.Widget = []
            self.ID = False
            self.Root = False
            self.Root_ID = False
            self.Element = False
            
            #Frame
            Fixture = self.Configure.Frame.Locate(100, 95, 0, 5)
            self.Frame = self.Global['Gluonix'].Scroll(self.Configure.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='#FFFFFF', Border_Size=0, Display=False)
            self.Frame.Config(Resize=False, Move=False)
            self.Frame.Create()
            self.Configure.Widget.append(self)
            
            #Name Label
            Fixture = self.Frame.Locate(25, 5, 3, 2)
            self.Name_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Name_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Name_Label.Config(Foreground='#000000', Value="Name:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Name_Label.Create()
            
            #Name Entry
            Fixture = self.Frame.Locate(60, 5, 28, 2)
            self.Name_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Name_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Name_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='left', Border_Size=1)
            self.Name_Entry.Bind(On_Key_Release=lambda E: self.Update_Name())
            self.Name_Entry.Create()
            
            #Background Label
            Fixture = self.Frame.Locate(25, 5, 3, 9)
            self.Background_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Background_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Background_Label.Config(Foreground='#000000', Value="Background:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Background_Label.Create()
            
            #Background Color
            Fixture = self.Frame.Locate(7, 5, 28, 9)
            self.Background_Color = self.Global['Gluonix'].Label(self.Frame)
            self.Background_Color.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Background_Color.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='w', Border_Size=1)
            self.Background_Color.Bind(On_Click=lambda E: self.Select_Color(self.Background_Color))
            self.Background_Color.Bind(On_Change=lambda : self.Update_Background())
            self.Background_Color.Create()
            
            #Background Check
            Fixture = self.Frame.Locate(7, 5, 37, 9)
            self.Background_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Background_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Background_Check.Config(Border_Size=0)
            self.Background_Check.Bind(On_Change=lambda : self.Update_Background())
            self.Background_Check.Create()
            
            #Foreground Label
            Fixture = self.Frame.Locate(25, 5, 53, 9)
            self.Foreground_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Foreground_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Foreground_Label.Config(Foreground='#000000', Value="Foreground:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Foreground_Label.Create()
            
            #Foreground Color
            Fixture = self.Frame.Locate(7, 5, 78, 9)
            self.Foreground_Color = self.Global['Gluonix'].Label(self.Frame)
            self.Foreground_Color.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Foreground_Color.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='w', Border_Size=1)
            self.Foreground_Color.Bind(On_Click=lambda E: self.Select_Color(self.Foreground_Color))
            self.Foreground_Color.Bind(On_Change=lambda : self.Update_Foreground())
            self.Foreground_Color.Create()
            
            #Border Color Label
            Fixture = self.Frame.Locate(25, 5, 3, 16)
            self.Border_Color_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Border_Color_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Border_Color_Label.Config(Foreground='#000000', Value="Border Color:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Border_Color_Label.Create()
            
            #Border Color
            Fixture = self.Frame.Locate(7, 5, 28, 16)
            self.Border_Color = self.Global['Gluonix'].Label(self.Frame)
            self.Border_Color.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Border_Color.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='w', Border_Size=1)
            self.Border_Color.Bind(On_Click=lambda E: self.Select_Color(self.Border_Color))
            self.Border_Color.Bind(On_Change=lambda : self.Update_Border_Color())
            self.Border_Color.Create()
            
            #Border_Size Label
            Fixture = self.Frame.Locate(25, 5, 3, 23)
            self.Border_Size_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Border_Size_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Border_Size_Label.Config(Foreground='#000000', Value="Border Size:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Border_Size_Label.Create()
            
            #Border_Size Entry
            Fixture = self.Frame.Locate(40, 5, 28, 23)
            self.Border_Size_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Border_Size_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Border_Size_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Border_Size_Entry.Bind(On_Key_Release=lambda E: self.Update_Border_Size())
            self.Border_Size_Entry.Create()
            
            #Alignment Label
            Fixture = self.Frame.Locate(25, 5, 3, 30)
            self.Alignment_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Alignment_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Alignment_Label.Config(Foreground='#000000', Value="Alignment:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Alignment_Label.Create()
            
            #Alignment Select
            Fixture = self.Frame.Locate(40, 5, 28, 30)
            self.Alignment_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Alignment_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Alignment_Select.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Alignment_Select.Add('Percentage')
            self.Alignment_Select.Add('Pixel')
            self.Alignment_Select.Bind(On_Change=lambda E: self.Update_Alignment())
            self.Alignment_Select.Create()
            
            #Alignment Lock
            Fixture = self.Frame.Locate(7, 5, 70, 30)
            self.Lock = False
            self.Lock_Image = {True: 'Lock_Closed', False: 'Lock_Open'}
            self.Alignment_Lock = self.Global['Gluonix'].Image(self.Frame)
            self.Alignment_Lock.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Alignment_Lock.Config(Border_Size=0)
            self.Alignment_Lock.Bind(On_Click=lambda E: self.Update_Lock())
            self.Alignment_Lock.Create()
            
            #Width Label
            Fixture = self.Frame.Locate(25, 5, 3, 37)
            self.Width_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Width_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Width_Label.Config(Foreground='#000000', Value="Width:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Width_Label.Create()
            
            #Width Entry
            Fixture = self.Frame.Locate(40, 5, 28, 37)
            self.Width_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Width_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Width_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Width_Entry.Bind(On_Key_Release=lambda E: self.Update_Width())
            self.Width_Entry.Create()
            
            #Height Label
            Fixture = self.Frame.Locate(25, 5, 3, 44)
            self.Height_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Height_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Height_Label.Config(Foreground='#000000', Value="Height:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Height_Label.Create()
            
            #Height Entry
            Fixture = self.Frame.Locate(40, 5, 28, 44)
            self.Height_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Height_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Height_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Height_Entry.Bind(On_Key_Release=lambda E: self.Update_Height())
            self.Height_Entry.Create()
            
            #Left Label
            Fixture = self.Frame.Locate(25, 5, 3, 51)
            self.Left_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Left_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Left_Label.Config(Foreground='#000000', Value="Left:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Left_Label.Create()
            
            #Left Entry
            Fixture = self.Frame.Locate(40, 5, 28, 51)
            self.Left_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Left_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Left_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Left_Entry.Bind(On_Key_Release=lambda E: self.Update_Left())
            self.Left_Entry.Create()
            
            #Top Label
            Fixture = self.Frame.Locate(25, 5, 3, 58)
            self.Top_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Top_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Top_Label.Config(Foreground='#000000', Value="Top:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Top_Label.Create()
            
            #Top Entry
            Fixture = self.Frame.Locate(40, 5, 28, 58)
            self.Top_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Top_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Top_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Top_Entry.Bind(On_Key_Release=lambda E: self.Update_Top())
            self.Top_Entry.Create()
            
            #Display Label
            Fixture = self.Frame.Locate(25, 5, 3, 65)
            self.Display_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Display_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Display_Label.Config(Foreground='#000000', Value="Display:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Display_Label.Create()
            
            #Display Check
            Fixture = self.Frame.Locate(7, 5, 27.7, 65)
            self.Display_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Display_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Display_Check.Config(Border_Size=0)
            self.Display_Check.Bind(On_Change=lambda : self.Update_Display())
            self.Display_Check.Create()
            
            #Resize Width Label
            Fixture = self.Frame.Locate(25, 5, 3, 72)
            self.Resize_Width_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Resize_Width_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Width_Label.Config(Foreground='#000000', Value="Resize Width:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Resize_Width_Label.Create()
            
            #Resize Width Check
            Fixture = self.Frame.Locate(7, 5, 27.7, 72)
            self.Resize_Width_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Resize_Width_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Width_Check.Config(Border_Size=0)
            self.Resize_Width_Check.Bind(On_Change=lambda : self.Update_Resize_Width())
            self.Resize_Width_Check.Create()
            
            #Resize Height Label
            Fixture = self.Frame.Locate(25, 5, 3, 79)
            self.Resize_Height_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Resize_Height_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Height_Label.Config(Foreground='#000000', Value="Resize Height:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Resize_Height_Label.Create()
            
            #Resize Height Check
            Fixture = self.Frame.Locate(7, 5, 27.7, 79)
            self.Resize_Height_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Resize_Height_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Height_Check.Config(Border_Size=0)
            self.Resize_Height_Check.Bind(On_Change=lambda : self.Update_Resize_Height())
            self.Resize_Height_Check.Create()
            
            #Move Left Label
            Fixture = self.Frame.Locate(25, 5, 3, 86)
            self.Move_Left_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Move_Left_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Move_Left_Label.Config(Foreground='#000000', Value="Move Left:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Move_Left_Label.Create()
            
            #Move Left Check
            Fixture = self.Frame.Locate(7, 5, 27.7, 86)
            self.Move_Left_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Move_Left_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Move_Left_Check.Config(Border_Size=0)
            self.Move_Left_Check.Bind(On_Change=lambda : self.Update_Move_Left())
            self.Move_Left_Check.Create()
            
            #Move Top Label
            Fixture = self.Frame.Locate(25, 5, 3, 93)
            self.Move_Top_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Move_Top_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Move_Top_Label.Config(Foreground='#000000', Value="Move Top:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Move_Top_Label.Create()
            
            #Move Top Check
            Fixture = self.Frame.Locate(7, 5, 27.7, 93)
            self.Move_Top_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Move_Top_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Move_Top_Check.Config(Border_Size=0)
            self.Move_Top_Check.Bind(On_Change=lambda : self.Update_Move_Top())
            self.Move_Top_Check.Create()
            
            #Anchor Label
            Fixture = self.Frame.Locate(25, 5, 3, 99)
            self.Anchor_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Anchor_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Anchor_Label.Config(Foreground='#000000', Value="Anchor:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Anchor_Label.Create()
            
            #Anchor Select
            Fixture = self.Frame.Locate(40, 5, 28, 99)
            self.Anchor_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Anchor_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Anchor_Select.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Anchor_Select.Add('center')
            self.Anchor_Select.Add('left')
            self.Anchor_Select.Add('right')
            self.Anchor_Select.Bind(On_Change=lambda E: self.Update_Anchor())
            self.Anchor_Select.Create()
            
            #Font Size Label
            Fixture = self.Frame.Locate(25, 5, 3, 106)
            self.Font_Size_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Font_Size_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Font_Size_Label.Config(Foreground='#000000', Value="Font Size:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Font_Size_Label.Create()
            
            #Font Size Entry
            Fixture = self.Frame.Locate(40, 5, 28, 106)
            self.Font_Size_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Font_Size_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Font_Size_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Font_Size_Entry.Bind(On_Key_Release=lambda E: self.Update_Font_Size())
            self.Font_Size_Entry.Create()
            
            #Font Weight Label
            Fixture = self.Frame.Locate(25, 5, 3, 113)
            self.Font_Weight_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Font_Weight_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Font_Weight_Label.Config(Foreground='#000000', Value="Font Weight:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Font_Weight_Label.Create()
            
            #Font Weight Select
            Fixture = self.Frame.Locate(40, 5, 28, 113)
            self.Font_Weight_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Font_Weight_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Font_Weight_Select.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Font_Weight_Select.Add('normal')
            self.Font_Weight_Select.Add('bold')
            self.Font_Weight_Select.Bind(On_Change=lambda E: self.Update_Font_Weight())
            self.Font_Weight_Select.Create()
            
            #Font Family Label
            Fixture = self.Frame.Locate(25, 5, 3, 120)
            self.Font_Family_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Font_Family_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Font_Family_Label.Config(Foreground='#000000', Value="Font Family:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Font_Family_Label.Create()
            
            #Font Family Select
            Fixture = self.Frame.Locate(40, 5, 28, 120)
            self.Font_Family_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Font_Family_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Font_Family_Select.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Font_Family_Select.Add('Arial')
            self.Font_Family_Select.Add('Courier')
            self.Font_Family_Select.Add('Courier New')
            self.Font_Family_Select.Add('Georgia')
            self.Font_Family_Select.Add('Lucida Console')
            self.Font_Family_Select.Add('Symbol')
            self.Font_Family_Select.Add('Times New Roman')
            self.Font_Family_Select.Add('Verdana')
            self.Font_Family_Select.Add('Comic Sans MS')
            self.Font_Family_Select.Add('Helvetica')
            self.Font_Family_Select.Bind(On_Change=lambda E: self.Update_Font_Family())
            self.Font_Family_Select.Create()
            
            #Update Scroll
            self.Frame.Update(self.Font_Family_Select)
            
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Load(self, ID=False):
        try:
            if ID!=self.ID:
                self.Root = False
                self.Root_ID = False
                self.Element = False
                self.ID = ID
                Widget_Data = self.Configure.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `ID`='{self.ID}'", Keys=True)
                if len(Widget_Data)==1:
                    Widget_Data = Widget_Data[0]
                    self.Root_ID = Widget_Data['Root']
                    self.Name_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Border_Size_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Width_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Height_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Left_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Top_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Name_Label.Set(Widget_Data['Type'])
                    self.Name_Entry.Set(Widget_Data['Name'])
                    if Widget_Data['Background']=='False':
                        self.Background_Check.Set(False)
                    else:
                        self.Background_Check.Set(True)
                        self.Background_Color.Config(Background=Widget_Data['Background'])
                    self.Foreground_Color.Config(Background=Widget_Data['Foreground'])
                    self.Border_Color.Config(Background=Widget_Data['Border_Color'])
                    self.Border_Size_Entry.Set(Widget_Data['Border_Size'])
                    self.Alignment_Select.Set(Widget_Data['Alignment'])
                    self.Lock = bool(Widget_Data['Lock'])
                    self.Alignment_Lock.Set(self.Global['Image'](self.Lock_Image[self.Lock]))
                    self.Width_Entry.Set(Widget_Data['Width'])
                    self.Height_Entry.Set(Widget_Data['Height'])
                    self.Left_Entry.Set(Widget_Data['Left'])
                    self.Top_Entry.Set(Widget_Data['Top'])
                    self.Display_Check.Set(bool(Widget_Data['Display']))
                    self.Resize_Width_Check.Set(bool(Widget_Data['Resize_Width']))
                    self.Resize_Height_Check.Set(bool(Widget_Data['Resize_Height']))
                    self.Move_Left_Check.Set(bool(Widget_Data['Move_Left']))
                    self.Move_Top_Check.Set(bool(Widget_Data['Move_Top']))
                    self.Anchor_Select.Set(Widget_Data['Align'])
                    self.Font_Weight_Select.Set(Widget_Data['Font_Weight'])
                    self.Font_Size_Entry.Set(Widget_Data['Font_Size'])
                    self.Font_Family_Select.Set(Widget_Data['Font_Family'])
                    self.Configure.Hide_All()
                    self.Configure.Current = self
                    self.Frame.Show()
                    Temp_Root = Widget_Data['Root']
                    Root = Temp_Root
                    while Temp_Root!='Root':
                        Widget_Data = self.Configure.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Temp_Root}'", Keys=True)
                        if len(Widget_Data)==1:
                            Widget_Data = Widget_Data[0]
                            Temp_Root = Widget_Data['Root']
                            Root = Temp_Root+'.'+Root
                        else:
                            break
                    Root_Split = Root.split('.')
                    Current = ''
                    for Each in Root_Split:
                        if Current:
                            Current = Current+'.'+Each
                        else:
                            Current = Each
                        if Current!='Root':
                            Temp_Root = self.Global['Custom'].Get_Attr_Class(self.Configure.Design, Current)
                            Temp_Root.Show()
                    self.Root = self.Global['Custom'].Get_Attr_Class(self.Configure.Design, Root)
                    self.Element = getattr(self.Root, self.ID)
                    self.Element.Show()
                else:
                    self.Global['Message'].Show('Error', 'Project Files Corrupted\nReopen Project')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        self.Global['Loading'].Hide()
            
    def Movement_Update(self):
        try:
            if self.ID:
                Widget_Data = self.Configure.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `ID`='{self.ID}'", Keys=True)
                Widget_Data = Widget_Data[0]
                self.Width_Entry.Set(Widget_Data['Width'])
                self.Height_Entry.Set(Widget_Data['Height'])
                self.Left_Entry.Set(Widget_Data['Left'])
                self.Top_Entry.Set(Widget_Data['Top'])
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Select_Color(self, Element):
        try:
            Color = colorchooser.askcolor(color=Element.Config_Get('Background')['Background'], title ="Choose Background Color")[1]
            if Color is not None:
                Element.Config(Background=Color)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Name(self):
        try:
            Name = self.Name_Entry.Get()
            TempName = self.Configure.Design.Database.Get(f"SELECT * FROM `Widget` WHERE (`ID`!='{self.ID}' AND `Root`=='{self.Root_ID}' AND `Name`='{Name}')")
            if self.Global['Custom'].Valid_Variable(Name) and len(TempName)==0:
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Name`='{Name}' WHERE `ID`='{self.ID}'")
                self.Configure.Design.Element.Tree.Edit(Name=f' {Name}')
                self.Name_Entry.Config(Border_Color='#000000', Border_Size=1)
            else:
                self.Name_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Background(self):
        try:
            if self.Background_Check.Get():
                Background = self.Background_Color.Config_Get('Background')['Background']
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Background`='{Background}' WHERE `ID`='{self.ID}'")
                if self.Element:
                    self.Element.Config(Background=Background)
            else:
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Background`='False' WHERE `ID`='{self.ID}'")
                if self.Element:
                    self.Element.Config(Background=False)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Foreground(self):
        try:
            Foreground = self.Foreground_Color.Config_Get('Background')['Background']
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Foreground`='{Foreground}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Config(Foreground=Foreground)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Border_Color(self):
        try:
            Border_Color = self.Border_Color.Config_Get('Background')['Background']
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Border_Color`='{Border_Color}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Config(Border_Color=Border_Color)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Border_Size(self):
        try:
            Border_Size = self.Border_Size_Entry.Get()
            if Border_Size.isdigit():
                Border_Size = int(Border_Size)
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Border_Size`='{Border_Size}' WHERE `ID`='{self.ID}'")
                self.Border_Size_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    self.Element.Config(Border_Size=Border_Size)
            else:
                self.Border_Size_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Alignment(self):
        try:
            Alignment = self.Alignment_Select.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Alignment`='{Alignment}' WHERE `ID`='{self.ID}'")
            if self.Element:
                Width = float(self.Width_Entry.Get())
                Height = float(self.Height_Entry.Get())
                Left = float(self.Left_Entry.Get())
                Top = float(self.Top_Entry.Get())
                if Alignment=='Percentage':
                    Fixture = self.Root.Locate(Width, Height, Left, Top)
                else:
                    Fixture = [Width, Height, Left, Top]
                self.Element.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Lock(self):
        try:
            self.Lock = not self.Lock
            self.Alignment_Lock.Set(self.Global['Image'](self.Lock_Image[self.Lock]))
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Lock`='{int(self.Lock)}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Lock = self.Lock
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Width(self):
        try:
            Width = self.Width_Entry.Get()
            if self.Global['Custom'].Is_Float(Width):
                Width = float(Width)
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Width`='{Width}' WHERE `ID`='{self.ID}'")
                self.Width_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    if self.Alignment_Select.Get()=='Percentage':
                        Fixture = self.Root.Locate(Width, 0, 0, 0)
                    else:
                        Fixture = [Width, 0, 0, 0]
                    self.Element.Config(Width=Fixture[0])
            else:
                self.Width_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Height(self):
        try:
            Height = self.Height_Entry.Get()
            if self.Global['Custom'].Is_Float(Height):
                Height = float(Height)
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Height`='{Height}' WHERE `ID`='{self.ID}'")
                self.Height_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    if self.Alignment_Select.Get()=='Percentage':
                        Fixture = self.Root.Locate(0, Height, 0, 0)
                    else:
                        Fixture = [0, Height, 0, 0]
                    self.Element.Config(Height=Fixture[1])
            else:
                self.Height_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Left(self):
        try:
            Left = self.Left_Entry.Get()
            if self.Global['Custom'].Is_Float(Left):
                Left = float(Left)
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Left`='{Left}' WHERE `ID`='{self.ID}'")
                self.Left_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    if self.Alignment_Select.Get()=='Percentage':
                        Fixture = self.Root.Locate(0, 0, Left, 0)
                    else:
                        Fixture = [0, 0, Left, 0]
                    self.Element.Config(Left=Fixture[2])
            else:
                self.Left_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Top(self):
        try:
            Top = self.Top_Entry.Get()
            if self.Global['Custom'].Is_Float(Top):
                Top = float(Top)
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Top`='{Top}' WHERE `ID`='{self.ID}'")
                self.Top_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    if self.Alignment_Select.Get()=='Percentage':
                        Fixture = self.Root.Locate(0, 0, 0, Top)
                    else:
                        Fixture = [0, 0, 0, Top]
                    self.Element.Config(Top=Fixture[3])
            else:
                self.Top_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Display(self):
        try:
            Display = self.Display_Check.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Display`='{int(Display)}' WHERE `ID`='{self.ID}'")
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Resize_Width(self):
        try:
            Resize_Width = self.Resize_Width_Check.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Resize_Width`='{int(Resize_Width)}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Config(Resize_Width=Resize_Width)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Resize_Height(self):
        try:
            Resize_Height = self.Resize_Height_Check.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Resize_Height`='{int(Resize_Height)}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Config(Resize_Height=Resize_Height)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Move_Left(self):
        try:
            Move_Left = self.Move_Left_Check.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Move_Left`='{int(Move_Left)}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Config(Move_Left=Move_Left)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Move_Top(self):
        try:
            Move_Top = self.Move_Top_Check.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Move_Top`='{int(Move_Top)}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Config(Move_Top=Move_Top)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Anchor(self):
        try:
            Anchor = self.Anchor_Select.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Align`='{Anchor}' WHERE `ID`='{self.ID}'")
            self.Element.Config(Align=Anchor)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Font_Weight(self):
        try:
            Font_Weight = self.Font_Weight_Select.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Font_Weight`='{Font_Weight}' WHERE `ID`='{self.ID}'")
            self.Element.Config(Font_Weight=Font_Weight)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Font_Size(self):
        try:
            Font_Size = self.Font_Size_Entry.Get()
            if Font_Size.isdigit():
                Font_Size = int(Font_Size)
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Font_Size`='{Font_Size}' WHERE `ID`='{self.ID}'")
                self.Font_Size_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    self.Element.Config(Font_Size=Font_Size)
            else:
                self.Font_Size_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Font_Family(self):
        try:
            Font_Family = self.Font_Family_Select.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Widget` SET `Font_Family`='{Font_Family}' WHERE `ID`='{self.ID}'")
            self.Element.Config(Font_Family=Font_Family)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


def Event_Bind(Widget, **Input):
    if 'Cursor_Hand' in Input:
        if Input['Cursor_Hand']:
            Widget.config(cursor="hand2")
    if 'Cursor_Loading' in Input:
        if Input['Cursor_Loading']:
            Widget.config(cursor="watch")
    if 'Cursor_Resize_Vertical' in Input:
        if Input['Cursor_Resize_Vertical']:
            Widget.config(cursor='sb_h_double_arrow')
    if 'Cursor_Resize_Horizontal' in Input:
        if Input['Cursor_Resize_Horizontal']:
            Widget.config(cursor='sb_v_double_arrow')
    if 'Cursor_Arrow' in Input:
        if Input['Cursor_Arrow']:
            Widget.config(cursor="")
    if 'Cursor' in Input:
        if Input['Cursor']:
            Widget.config(cursor=Input['Cursor'])
    if "On_Configure" in Input:
        Widget.bind("<Configure>", Input["On_Configure"])
    if "On_Destroy" in Input:
        Widget.bind("<Destroy>", Input["On_Destroy"])
    if "On_Expose" in Input:
        Widget.bind("<Expose>", Input["On_Expose"])
    if "On_Visibility" in Input:
        Widget.bind("<Visibility>", Input["On_Visibility"])
    if "On_Motion" in Input:
        Widget.bind("<Motion>", Input["On_Motion"])
    if "On_Click" in Input:
        Widget.bind("<ButtonPress-1>", Input["On_Click"])
    if "On_Release" in Input:
        Widget.bind("<ButtonRelease-1>", Input["On_Release"])
    if "On_Double_Click" in Input:
        Widget.bind("<Double-1>", Input["On_Double_Click"])
    if "On_Triple_Click" in Input:
        Widget.bind("<Triple-1>", Input["On_Triple_Click"])
    if "On_Middle_Click" in Input:
        Widget.bind("<ButtonPress-2>", Input["On_Middle_Click"])
    if "On_Middle_Release" in Input:
        Widget.bind("<ButtonRelease-2>", Input["On_Middle_Release"])
    if "On_Middle_Double_Click" in Input:
        Widget.bind("<Double-2>", Input["On_Middle_Double_Click"])
    if "On_Middle_Triple_Click" in Input:
        Widget.bind("<Triple-2>", Input["On_Middle_Triple_Click"])
    if "On_Right_Click" in Input:
        Widget.bind("<ButtonPress-3>", Input["On_Right_Click"])
    if "On_Right_Release" in Input:
        Widget.bind("<ButtonRelease-3>", Input["On_Right_Release"])
    if "On_Right_Double_Click" in Input:
        Widget.bind("<Double-3>", Input["On_Right_Double_Click"])
    if "On_Right_Triple_Click" in Input:
        Widget.bind("<Triple-3>", Input["On_Right_Triple_Click"])
    if "On_Drag" in Input:
        Widget.bind("<B1-Motion>", Input["On_Drag"])
    if "On_Middle_Drag" in Input:
        Widget.bind("<B2-Motion>", Input["On_Middle_Drag"])
    if "On_Right_Drag" in Input:
        Widget.bind("<B3-Motion>", Input["On_Right_Drag"])
    if "On_Mouse_Wheel" in Input:
        Widget.bind("<MouseWheel>", Input["On_Mouse_Wheel"])
    if "On_Hover_In" in Input:
        Widget.bind("<Enter>", Input["On_Hover_In"])
    if "On_Hover_Out" in Input:
        Widget.bind("<Leave>", Input["On_Hover_Out"])
    if "On_Key" in Input:
        Widget.bind("<KeyPress>", Input["On_Key"])
    if "On_Key_Release" in Input:
        Widget.bind("<KeyRelease>", Input["On_Key_Release"])
    if "On_Focus_In" in Input:
        Widget.bind("<FocusIn>", Input["On_Focus_In"])
    if "On_Focus_Out" in Input:
        Widget.bind("<FocusOut>", Input["On_Focus_Out"])
    if "On_Map" in Input:
        Widget.bind("<Map>", Input["On_Map"])
    if "On_Unmap" in Input:
        Widget.bind("<Unmap>", Input["On_Unmap"])
    if "On_Copy" in Input:
        Widget.bind("<<Copy>>", Input["On_Copy"])
    if "On_Cut" in Input:
        Widget.bind("<<Cut>>", Input["On_Cut"])
    if "On_Paste" in Input:
        Widget.bind("<<Paste>>", Input["On_Paste"])
    if "On_Undo" in Input:
        Widget.bind("<<Undo>>", Input["On_Undo"])
    if "On_Redo" in Input:
        Widget.bind("<<Redo>>", Input["On_Redo"])
    if "On_Control_Click" in Input:
        Widget.bind("<Control-Button-1>", Input["On_Control_Click"])
    if "On_Control_Release" in Input:
        Widget.bind("<Control-ButtonRelease-1>", Input["On_Control_Release"])
    if "On_Control_Double_Click" in Input:
        Widget.bind("<Control-Double-1>", Input["On_Control_Double_Click"])
    if "On_Control_Triple_Click" in Input:
        Widget.bind("<Control-Triple-1>", Input["On_Control_Triple_Click"])
    if "On_Control_Middle_Click" in Input:
        Widget.bind("<Control-Button-2>", Input["On_Control_Middle_Click"])
    if "On_Control_Middle_Release" in Input:
        Widget.bind("<Control-ButtonRelease-2>", Input["On_Control_Middle_Release"])
    if "On_Control_Middle_Double_Click" in Input:
        Widget.bind("<Control-Double-2>", Input["On_Control_Middle_Double_Click"])
    if "On_Control_Middle_Triple_Click" in Input:
        Widget.bind("<Control-Triple-2>", Input["On_Control_Middle_Triple_Click"])
    if "On_Control_Right_Click" in Input:
        Widget.bind("<Control-Button-3>", Input["On_Control_Right_Click"])
    if "On_Control_Right_Release" in Input:
        Widget.bind("<Control-ButtonRelease-3>", Input["On_Control_Right_Release"])
    if "On_Control_Right_Double_Click" in Input:
        Widget.bind("<Control-Double-3>", Input["On_Control_Right_Double_Click"])
    if "On_Control_Right_Triple_Click" in Input:
        Widget.bind("<Control-Triple-3>", Input["On_Control_Right_Triple_Click"])
    if "On_Control_Drag" in Input:
        Widget.bind("<Control-B1-Motion>", Input["On_Control_Drag"])
    if "On_Control_Middle_Drag" in Input:
        Widget.bind("<Control-B2-Motion>", Input["On_Control_Middle_Drag"])
    if "On_Control_Right_Drag" in Input:
        Widget.bind("<Control-B3-Motion>", Input["On_Control_Right_Drag"])
    if "On_Control_Mouse_Wheel" in Input:
        Widget.bind("<Control-MouseWheel>", Input["On_Control_Mouse_Wheel"])
    if "On_Control_Hover_In" in Input:
        Widget.bind("<Control-Enter>", Input["On_Control_Hover_In"])
    if "On_Control_Hover_Out" in Input:
        Widget.bind("<Control-Leave>", Input["On_Control_Hover_Out"])
    if "On_Alt_Click" in Input:
        Widget.bind("<Alt-Button-1>", Input["On_Alt_Click"])
    if "On_Alt_Release" in Input:
        Widget.bind("<Alt-ButtonRelease-1>", Input["On_Alt_Release"])
    if "On_Alt_Double_Click" in Input:
        Widget.bind("<Alt-Double-1>", Input["On_Alt_Double_Click"])
    if "On_Alt_Triple_Click" in Input:
        Widget.bind("<Alt-Triple-1>", Input["On_Alt_Triple_Click"])
    if "On_Alt_Middle_Click" in Input:
        Widget.bind("<Alt-Button-2>", Input["On_Alt_Middle_Click"])
    if "On_Alt_Middle_Release" in Input:
        Widget.bind("<Alt-ButtonRelease-2>", Input["On_Alt_Middle_Release"])
    if "On_Alt_Middle_Double_Click" in Input:
        Widget.bind("<Alt-Double-2>", Input["On_Alt_Middle_Double_Click"])
    if "On_Alt_Middle_Triple_Click" in Input:
        Widget.bind("<Alt-Triple-2>", Input["On_Alt_Middle_Triple_Click"])
    if "On_Alt_Right_Click" in Input:
        Widget.bind("<Alt-Button-3>", Input["On_Alt_Right_Click"])
    if "On_Alt_Right_Release" in Input:
        Widget.bind("<Alt-ButtonRelease-3>", Input["On_Alt_Right_Release"])
    if "On_Alt_Right_Double_Click" in Input:
        Widget.bind("<Alt-Double-3>", Input["On_Alt_Right_Double_Click"])
    if "On_Alt_Right_Triple_Click" in Input:
        Widget.bind("<Alt-Triple-3>", Input["On_Alt_Right_Triple_Click"])
    if "On_Alt_Drag" in Input:
        Widget.bind("<Alt-B1-Motion>", Input["On_Alt_Drag"])
    if "On_Alt_Middle_Drag" in Input:
        Widget.bind("<Alt-B2-Motion>", Input["On_Alt_Middle_Drag"])
    if "On_Alt_Right_Drag" in Input:
        Widget.bind("<Alt-B3-Motion>", Input["On_Alt_Right_Drag"])
    if "On_Alt_Mouse_Wheel" in Input:
        Widget.bind("<Alt-MouseWheel>", Input["On_Alt_Mouse_Wheel"])
    if "On_Alt_Hover_In" in Input:
        Widget.bind("<Alt-Enter>", Input["On_Alt_Hover_In"])
    if "On_Alt_Hover_Out" in Input:
        Widget.bind("<Alt-Leave>", Input["On_Alt_Hover_Out"])
    if "On_Shift_Click" in Input:
        Widget.bind("<Shift-Button-1>", Input["On_Shift_Click"])
    if "On_Shift_Release" in Input:
        Widget.bind("<Shift-ButtonRelease-1>", Input["On_Shift_Release"])
    if "On_Shift_Double_Click" in Input:
        Widget.bind("<Shift-Double-1>", Input["On_Shift_Double_Click"])
    if "On_Shift_Triple_Click" in Input:
        Widget.bind("<Shift-Triple-1>", Input["On_Shift_Triple_Click"])
    if "On_Shift_Middle_Click" in Input:
        Widget.bind("<Shift-Button-2>", Input["On_Shift_Middle_Click"])
    if "On_Shift_Middle_Release" in Input:
        Widget.bind("<Shift-ButtonRelease-2>", Input["On_Shift_Middle_Release"])
    if "On_Shift_Middle_Double_Click" in Input:
        Widget.bind("<Shift-Double-2>", Input["On_Shift_Middle_Double_Click"])
    if "On_Shift_Middle_Triple_Click" in Input:
        Widget.bind("<Shift-Triple-2>", Input["On_Shift_Middle_Triple_Click"])
    if "On_Shift_Right_Click" in Input:
        Widget.bind("<Shift-Button-3>", Input["On_Shift_Right_Click"])
    if "On_Shift_Right_Release" in Input:
        Widget.bind("<Shift-ButtonRelease-3>", Input["On_Shift_Right_Release"])
    if "On_Shift_Right_Double_Click" in Input:
        Widget.bind("<Shift-Double-3>", Input["On_Shift_Right_Double_Click"])
    if "On_Shift_Right_Triple_Click" in Input:
        Widget.bind("<Shift-Triple-3>", Input["On_Shift_Right_Triple_Click"])
    if "On_Shift_Drag" in Input:
        Widget.bind("<Shift-B1-Motion>", Input["On_Shift_Drag"])
    if "On_Shift_Middle_Drag" in Input:
        Widget.bind("<Shift-B2-Motion>", Input["On_Shift_Middle_Drag"])
    if "On_Shift_Right_Drag" in Input:
        Widget.bind("<Shift-B3-Motion>", Input["On_Shift_Right_Drag"])
    if "On_Shift_Mouse_Wheel" in Input:
        Widget.bind("<Shift-MouseWheel>", Input["On_Shift_Mouse_Wheel"])
    if "On_Shift_Hover_In" in Input:
        Widget.bind("<Shift-Enter>", Input["On_Shift_Hover_In"])
    if "On_Shift_Hover_Out" in Input:
        Widget.bind("<Shift-Leave>", Input["On_Shift_Hover_Out"])
        
def Event_Bind_Canvas(Widget, Item, **Input):
    if "On_Motion" in Input:
        Widget.tag_bind(Item, "<Motion>", Input["On_Motion"])
    if "On_Click" in Input:
        Widget.tag_bind(Item, "<ButtonPress-1>", Input["On_Click"])
    if "On_Release" in Input:
        Widget.tag_bind(Item, "<ButtonRelease-1>", Input["On_Release"])
    if "On_Double_Click" in Input:
        Widget.tag_bind(Item, "<Double-1>", Input["On_Double_Click"])
    if "On_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Triple-1>", Input["On_Triple_Click"])
    if "On_Middle_Click" in Input:
        Widget.tag_bind(Item, "<ButtonPress-2>", Input["On_Middle_Click"])
    if "On_Middle_Release" in Input:
        Widget.tag_bind(Item, "<ButtonRelease-2>", Input["On_Middle_Release"])
    if "On_Middle_Double_Click" in Input:
        Widget.tag_bind(Item, "<Double-2>", Input["On_Middle_Double_Click"])
    if "On_Middle_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Triple-2>", Input["On_Middle_Triple_Click"])
    if "On_Right_Click" in Input:
        Widget.tag_bind(Item, "<ButtonPress-3>", Input["On_Right_Click"])
    if "On_Right_Release" in Input:
        Widget.tag_bind(Item, "<ButtonRelease-3>", Input["On_Right_Release"])
    if "On_Right_Double_Click" in Input:
        Widget.tag_bind(Item, "<Double-3>", Input["On_Right_Double_Click"])
    if "On_Right_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Triple-3>", Input["On_Right_Triple_Click"])
    if "On_Drag" in Input:
        Widget.tag_bind(Item, "<B1-Motion>", Input["On_Drag"])
    if "On_Middle_Drag" in Input:
        Widget.tag_bind(Item, "<B2-Motion>", Input["On_Middle_Drag"])
    if "On_Right_Drag" in Input:
        Widget.tag_bind(Item, "<B3-Motion>", Input["On_Right_Drag"])
    if "On_Mouse_Wheel" in Input:
        Widget.tag_bind(Item, "<MouseWheel>", Input["On_Mouse_Wheel"])
    if "On_Hover_In" in Input:
        Widget.tag_bind(Item, "<Enter>", Input["On_Hover_In"])
    if "On_Hover_Out" in Input:
        Widget.tag_bind(Item, "<Leave>", Input["On_Hover_Out"])
    if "On_Key" in Input:
        Widget.tag_bind(Item, "<KeyPress>", Input["On_Key"])
    if "On_Key_Release" in Input:
        Widget.tag_bind(Item, "<KeyRelease>", Input["On_Key_Release"])
    if "On_Focus_Out" in Input:
        Widget.tag_bind(Item, "<FocusIn>", Input["On_Focus_Out"])
    if "On_Focus_Out" in Input:
        Widget.tag_bind(Item, "<FocusOut>", Input["On_Focus_Out"])
    if "On_Map" in Input:
        Widget.tag_bind(Item, "<Map>", Input["On_Map"])
    if "On_Unmap" in Input:
        Widget.tag_bind(Item, "<Unmap>", Input["On_Unmap"])
    if "On_Copy" in Input:
        Widget.tag_bind(Item, "<<Copy>>", Input["On_Copy"])
    if "On_Cut" in Input:
        Widget.tag_bind(Item, "<<Cut>>", Input["On_Cut"])
    if "On_Paste" in Input:
        Widget.tag_bind(Item, "<<Paste>>", Input["On_Paste"])
    if "On_Undo" in Input:
        Widget.tag_bind(Item, "<<Undo>>", Input["On_Undo"])
    if "On_Redo" in Input:
        Widget.tag_bind(Item, "<<Redo>>", Input["On_Redo"])
    if "On_Control_Click" in Input:
        Widget.tag_bind(Item, "<Control-Button-1>", Input["On_Control_Click"])
    if "On_Control_Release" in Input:
        Widget.tag_bind(Item, "<Control-ButtonRelease-1>", Input["On_Control_Release"])
    if "On_Control_Double_Click" in Input:
        Widget.tag_bind(Item, "<Control-Double-1>", Input["On_Control_Double_Click"])
    if "On_Control_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Control-Triple-1>", Input["On_Control_Triple_Click"])
    if "On_Control_Middle_Click" in Input:
        Widget.tag_bind(Item, "<Control-Button-2>", Input["On_Control_Middle_Click"])
    if "On_Control_Middle_Release" in Input:
        Widget.tag_bind(Item, "<Control-ButtonRelease-2>", Input["On_Control_Middle_Release"])
    if "On_Control_Middle_Double_Click" in Input:
        Widget.tag_bind(Item, "<Control-Double-2>", Input["On_Control_Middle_Double_Click"])
    if "On_Control_Middle_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Control-Triple-2>", Input["On_Control_Middle_Triple_Click"])
    if "On_Control_Right_Click" in Input:
        Widget.tag_bind(Item, "<Control-Button-3>", Input["On_Control_Right_Click"])
    if "On_Control_Right_Release" in Input:
        Widget.tag_bind(Item, "<Control-ButtonRelease-3>", Input["On_Control_Right_Release"])
    if "On_Control_Right_Double_Click" in Input:
        Widget.tag_bind(Item, "<Control-Double-3>", Input["On_Control_Right_Double_Click"])
    if "On_Control_Right_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Control-Triple-3>", Input["On_Control_Right_Triple_Click"])
    if "On_Control_Drag" in Input:
        Widget.tag_bind(Item, "<Control-B1-Motion>", Input["On_Control_Drag"])
    if "On_Control_Middle_Drag" in Input:
        Widget.tag_bind(Item, "<Control-B2-Motion>", Input["On_Control_Middle_Drag"])
    if "On_Control_Right_Drag" in Input:
        Widget.tag_bind(Item, "<Control-B3-Motion>", Input["On_Control_Right_Drag"])
    if "On_Control_Mouse_Wheel" in Input:
        Widget.tag_bind(Item, "<Control-MouseWheel>", Input["On_Control_Mouse_Wheel"])
    if "On_Control_Hover_In" in Input:
        Widget.tag_bind(Item, "<Control-Enter>", Input["On_Control_Hover_In"])
    if "On_Control_Hover_Out" in Input:
        Widget.tag_bind(Item, "<Control-Leave>", Input["On_Control_Hover_Out"])
    if "On_Alt_Click" in Input:
        Widget.tag_bind(Item, "<Alt-Button-1>", Input["On_Alt_Click"])
    if "On_Alt_Release" in Input:
        Widget.tag_bind(Item, "<Alt-ButtonRelease-1>", Input["On_Alt_Release"])
    if "On_Alt_Double_Click" in Input:
        Widget.tag_bind(Item, "<Alt-Double-1>", Input["On_Alt_Double_Click"])
    if "On_Alt_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Alt-Triple-1>", Input["On_Alt_Triple_Click"])
    if "On_Alt_Middle_Click" in Input:
        Widget.tag_bind(Item, "<Alt-Button-2>", Input["On_Alt_Middle_Click"])
    if "On_Alt_Middle_Release" in Input:
        Widget.tag_bind(Item, "<Alt-ButtonRelease-2>", Input["On_Alt_Middle_Release"])
    if "On_Alt_Middle_Double_Click" in Input:
        Widget.tag_bind(Item, "<Alt-Double-2>", Input["On_Alt_Middle_Double_Click"])
    if "On_Alt_Middle_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Alt-Triple-2>", Input["On_Alt_Middle_Triple_Click"])
    if "On_Alt_Right_Click" in Input:
        Widget.tag_bind(Item, "<Alt-Button-3>", Input["On_Alt_Right_Click"])
    if "On_Alt_Right_Release" in Input:
        Widget.tag_bind(Item, "<Alt-ButtonRelease-3>", Input["On_Alt_Right_Release"])
    if "On_Alt_Right_Double_Click" in Input:
        Widget.tag_bind(Item, "<Alt-Double-3>", Input["On_Alt_Right_Double_Click"])
    if "On_Alt_Right_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Alt-Triple-3>", Input["On_Alt_Right_Triple_Click"])
    if "On_Alt_Drag" in Input:
        Widget.tag_bind(Item, "<Alt-B1-Motion>", Input["On_Alt_Drag"])
    if "On_Alt_Middle_Drag" in Input:
        Widget.tag_bind(Item, "<Alt-B2-Motion>", Input["On_Alt_Middle_Drag"])
    if "On_Alt_Right_Drag" in Input:
        Widget.tag_bind(Item, "<Alt-B3-Motion>", Input["On_Alt_Right_Drag"])
    if "On_Alt_Mouse_Wheel" in Input:
        Widget.tag_bind(Item, "<Alt-MouseWheel>", Input["On_Alt_Mouse_Wheel"])
    if "On_Alt_Hover_In" in Input:
        Widget.tag_bind(Item, "<Alt-Enter>", Input["On_Alt_Hover_In"])
    if "On_Alt_Hover_Out" in Input:
        Widget.tag_bind(Item, "<Alt-Leave>", Input["On_Alt_Hover_Out"])
    if "On_Shift_Click" in Input:
        Widget.tag_bind(Item, "<Shift-Button-1>", Input["On_Shift_Click"])
    if "On_Shift_Release" in Input:
        Widget.tag_bind(Item, "<Shift-ButtonRelease-1>", Input["On_Shift_Release"])
    if "On_Shift_Double_Click" in Input:
        Widget.tag_bind(Item, "<Shift-Double-1>", Input["On_Shift_Double_Click"])
    if "On_Shift_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Shift-Triple-1>", Input["On_Shift_Triple_Click"])
    if "On_Shift_Middle_Click" in Input:
        Widget.tag_bind(Item, "<Shift-Button-2>", Input["On_Shift_Middle_Click"])
    if "On_Shift_Middle_Release" in Input:
        Widget.tag_bind(Item, "<Shift-ButtonRelease-2>", Input["On_Shift_Middle_Release"])
    if "On_Shift_Middle_Double_Click" in Input:
        Widget.tag_bind(Item, "<Shift-Double-2>", Input["On_Shift_Middle_Double_Click"])
    if "On_Shift_Middle_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Shift-Triple-2>", Input["On_Shift_Middle_Triple_Click"])
    if "On_Shift_Right_Click" in Input:
        Widget.tag_bind(Item, "<Shift-Button-3>", Input["On_Shift_Right_Click"])
    if "On_Shift_Right_Release" in Input:
        Widget.tag_bind(Item, "<Shift-ButtonRelease-3>", Input["On_Shift_Right_Release"])
    if "On_Shift_Right_Double_Click" in Input:
        Widget.tag_bind(Item, "<Shift-Double-3>", Input["On_Shift_Right_Double_Click"])
    if "On_Shift_Right_Triple_Click" in Input:
        Widget.tag_bind(Item, "<Shift-Triple-3>", Input["On_Shift_Right_Triple_Click"])
    if "On_Shift_Drag" in Input:
        Widget.tag_bind(Item, "<Shift-B1-Motion>", Input["On_Shift_Drag"])
    if "On_Shift_Middle_Drag" in Input:
        Widget.tag_bind(Item, "<Shift-B2-Motion>", Input["On_Shift_Middle_Drag"])
    if "On_Shift_Right_Drag" in Input:
        Widget.tag_bind(Item, "<Shift-B3-Motion>", Input["On_Shift_Right_Drag"])
    if "On_Shift_Mouse_Wheel" in Input:
        Widget.tag_bind(Item, "<Shift-MouseWheel>", Input["On_Shift_Mouse_Wheel"])
    if "On_Shift_Hover_In" in Input:
        Widget.tag_bind(Item, "<Shift-Enter>", Input["On_Shift_Hover_In"])
    if "On_Shift_Hover_Out" in Input:
        Widget.tag_bind(Item, "<Shift-Leave>", Input["On_Shift_Hover_Out"])


################################################################################################################################
#Loading
################################################################################################################################
import time
import _thread
import inspect

class Loading:
    def __init__(self, Global):
        try:
            self.Global = Global
            self.Enabled = False
            #Frame
            Fixture = self.Global['GUI'].Locate(25, 15, 73.5, 82.5)
            self.Frame = self.Global['Gluonix'].Frame(self.Global['GUI'])
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='white', Border_Size=1, Display=False)
            self.Frame.Create()
            self.Global['Widget'].append(self.Frame)
                #Image
            Fixture = self.Frame.Locate(20, 96, 7, 2)
            self.Image = self.Global['Gluonix'].Image(self.Frame)
            self.Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Image.Config(Path=self.Global['Image']('Loading'), Border_Size=0)
            self.Image.Create()
                #Label
            Fixture = self.Frame.Locate(67, 96, 31, 2)
            self.Label = self.Global['Gluonix'].Label(self.Frame)
            self.Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label.Config(Foreground='black', Value="Loading Please Wait...", Font_Size=10, Font_Weight='normal', Align='center', Border_Size=0)
            self.Label.Create()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))

    def Show(self):
        try:
            self.Enabled = True
            _thread.start_new_thread(self.Load, ())
            self.Frame.Show()
            self.Global['GUI'].Bind(Cursor_Loading=True)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Hide(self, Delay=False):
        try:
            if Delay:
                self.Global['GUI'].After(Delay*1000, lambda: self.Hide(Delay=False))
            else:
                self.Enabled = False
                self.Frame.Hide()
            self.Global['GUI'].Bind(Cursor_Arrow=True)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
                    
    def Load(self):
        while True:
            try:
                if self.Enabled:
                    try:
                        self.Image.Rotate(-5)
                        time.sleep(0.05)
                    except Exception as E:
                        time.sleep(1)
                        self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
                else:
                    break
            except Exception:
                break
                        


################################################################################################################################
#Menu
################################################################################################################################
import inspect

#Program
from .N_Panel_Overview import Overview

class Panel:
    def __init__(self, Global, Home):
        try:
            self.Global = Global
            self.Home = Home
            self.Widget = []
            
            Fixture = self.Home.Frame.Locate(80, 100, 20, 0)
            self.Frame = self.Global['Gluonix'].Scroll(self.Home.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Border_Size=0, Display=True)
            self.Frame.Config(Resize=True, Move=True)
            self.Frame.Create()
            
            #Overview
            self.Overview = Overview(self.Global, self)

        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


import os
import subprocess
import hashlib
import keyword
import re
import builtins

    #Get System UUID
def UUID():
    if os.name == 'nt':
        return subprocess.check_output('wmic csproduct get uuid').decode().split('\n')[1].strip()
    else:
        Output = subprocess.check_output(['sudo', 'dmidecode', '-s', 'system-uuid'], stderr=subprocess.STDOUT)
        return Output.decode().strip()
    
    #Check Empty Folder
def Is_Empty(Path):
    return len(os.listdir(Path)) == 0
    
    #MD5 Hash Generator
def MD5(String):
    return hashlib.md5(String.encode('utf-8')).hexdigest()
    
    #String Is Float
def Is_Float(String):
    try:
        float(String)
        return True
    except:
        return False
    
    #String Is Int
def Is_Int(String):
    try:
        int(String)
        return True
    except:
        return False
        
    #Get Attribute
def Get_Attr_Class(Object, Attr):
    Attributes = Attr.split('.')
    for Attribute in Attributes:
        Object = getattr(Object, Attribute)
    return Object
    
    #Valid Variable
def Valid_Variable(Name):
    Custom = ['Error', 'Nothing', 'Maximize', 'Restart',  'Close',  'Hide', 'Show', 'Display', 'Save', 'Start', 'Event', 'Event_Runner', 'After', 'Screen', 'Full_Screen', 'Widget', 'Bind', 'Config_Get', 'Config', 'Add_Menu', 'Add_Sub_Manu', 'Add_Separator', 'Folder', 'File', 'Grab', 'Position', 'Locate', 'Locate_Reverse', 'Locate_Fullscreen', 'Create', 'Copy', 'Delete', 'Clear', 'Adjustment', 'Relocate', 'Resize', 'Top', 'Reset', 'Scroll', 'Bind_Item', 'Update', 'Update_All', 'Hide_Item', 'Show_Item', 'Delete_Item', 'Delete_All', 'Find_Near', 'Find_Overlap', 'Line', 'Polyline', 'Circle', 'Rectangle', 'Oval', 'Polygon', 'Text', 'Image', 'Border', 'Compound', 'Label', 'Open', 'Frame', 'Canvas', 'Scroll']
    if not Name:
        return False
    if Name in Custom:
        return False
    if keyword.iskeyword(Name):
        return False
    if Name in dir(builtins):
        return False
    if Name.startswith('_') or Name.endswith('_'):
        return False
    if Name[0].isdigit():
        return False
    valid_pattern = re.compile(r'^[A-Za-z0-9_]+$')
    if not valid_pattern.match(Name):
        return False
    if '__' in Name:
        return False
    return True


################################################################################################################################
#Element
################################################################################################################################
import inspect
import os
import shutil
import random
import string
import time
import traceback

class Element:
    def __init__(self, Global, Design):
        try:
            self.Global = Global
            self.Design = Design
            self.Widget = []
            self.Grid_Lock = True
            self.Copy_ID = False
            self.Current = False
            self.Dragging = False
            self.Current = False
            self._Coord = {'X': 0, 'Y': 0}
            self.Element_Fixture = {}
            
            #Frame
            Fixture = self.Design.Frame.Locate(29.5, 90, 30, 10)
            self.Frame = self.Global['Gluonix'].Frame(self.Design.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Frame.Config(Resize=False, Move=False)
            self.Frame.Create()
            self.Design.Widget.append(self.Frame)
            
            #Label
            Fixture = self.Frame.Locate(30, 3, 0, 1)
            self.Label = self.Global['Gluonix'].Label(self.Frame)
            self.Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Label.Config(Foreground='#000000', Font_Size=16, Font_Weight='normal', Align='w')
            self.Label.Config(Resize=False, Move=False)
            self.Label.Set(' PROJECT')
            self.Label.Create()
            
            #Grid
            Fixture = self.Frame.Locate(10, 4.7, 30, 0)
            self.Grid_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Grid_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Grid_Image.Config(Path=self.Global['Image']('Grid_Lock'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Grid_Image.Bind(On_Click=lambda E: self.Grid())
            self.Grid_Image.Create()
            
            #Paste
            Fixture = self.Frame.Locate(10, 4.7, 60, 0)
            self.Paste_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Paste_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Paste_Image.Config(Path=self.Global['Image']('Paste'), Border_Size=0, Hand_Cursor=True, Display=False)
            self.Paste_Image.Bind(On_Click=lambda E: self.Paste())
            self.Paste_Image.Create()
            
            #Copy
            Fixture = self.Frame.Locate(10, 5, 70, 0)
            self.Copy_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Copy_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Copy_Image.Config(Path=self.Global['Image']('Copy'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Copy_Image.Bind(On_Click=lambda E: self.Copy())
            self.Copy_Image.Create()
            
            #Export
            Fixture = self.Frame.Locate(10, 3.8, 80, 0.5)
            self.Export_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Export_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Export_Image.Config(Path=self.Global['Image']('Export'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Export_Image.Bind(On_Click=lambda E: self.Export())
            self.Export_Image.Create()
            
            #Delete
            Fixture = self.Frame.Locate(10, 5, 90, 0)
            self.Delete_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Image.Config(Path=self.Global['Image']('Delete'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Delete_Image.Bind(On_Click=lambda E: self.Show_Delete())
            self.Delete_Image.Create()
            
            #Delete Comfirm
            Fixture = self.Frame.Locate(10, 5, 79, 0)
            self.Delete_Confirm_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Confirm_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Confirm_Image.Config(Path=self.Global['Image']('Success'), Border_Size=0, Hand_Cursor=True, Display=False)
            self.Delete_Confirm_Image.Bind(On_Click=lambda E: self.Delete())
            self.Delete_Confirm_Image.Create()
            
            #Delete Cancel
            Fixture = self.Frame.Locate(10, 5, 90, 0)
            self.Delete_Cancel_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Cancel_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Cancel_Image.Config(Path=self.Global['Image']('Close'), Border_Size=0, Hand_Cursor=True, Display=False)
            self.Delete_Cancel_Image.Bind(On_Click=lambda E: self.Hide_Delete())
            self.Delete_Cancel_Image.Create()
            
            #Tree
            Fixture = self.Frame.Locate(100, 95, 0, 5)
            self.Tree = self.Global['Gluonix'].Tree(self.Frame)
            self.Tree.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Tree.Config(Background='#FFFFFF', Border_Size=0, Scroll_Width=Fixture[0]*3, Display=True)
            self.Tree.Config(Foreground='#000000', Font_Size=16, Font_Weight='normal')
            self.Tree.Config(Resize=False, Move=False)
            self.Tree.Bind(On_Release=lambda E: self.Reset_All(), On_Double_Click=lambda E: self.Get_Current(), On_Right_Click=lambda E: self.Get_Current())
            self.Tree.Create()
                   
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Load(self):
        try:
            self.Load_Child(self.Parent)
            self.Tree.Expand(getattr(self, self.Parent))
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Load_Child(self, Parent='Root'):
        try:
            Widgets = self.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `Root`='{Parent}'", Keys=True)
            for Widget in Widgets:
                Stock = getattr(self.Design.Stock, f"Stock_{Widget['Type']}")
                Stock.Create(Widget['ID'])
                Root = getattr(self, Widget['Root'])
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=Widget['Name'], Parent=Root, Value=[Widget['ID'], Widget['Type'], 0], Path=Image))
            Widgets = self.Design.Database.Get(f"SELECT * FROM `Item` WHERE `Root`='{Parent}'", Keys=True)
            for Widget in Widgets:
                Stock = getattr(self.Design.Stock, f"Stock_{Widget['Type']}")
                Stock.Create(Widget['ID'])
                Root = getattr(self, Widget['Root'])
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=Widget['Name'], Parent=Root, Value=[Widget['ID'], Widget['Type'], 0], Path=Image))
            Widgets = self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `Root`='{Parent}'", Keys=True)
            for Widget in Widgets:
                Stock = getattr(self.Design.Stock, f"Stock_{Widget['Type']}")
                Stock.Create(Widget['ID'])
                Root = getattr(self, Widget['Root'])
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=Widget['Name'], Parent=Root, Value=[Widget['ID'], Widget['Type'], Widget['Level']], Path=Image))
                self.Load_Child(Widget['ID'])
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Intractive(self, Element):
        try:
            Element.Bind(On_Click=lambda E: self.Drag_Start(E, Element), On_Drag=lambda E: self.Drag(E, Element), On_Release=lambda E: self.Drag_Release(E, Element))
            Element.Bind(On_Right_Click=lambda E: self.Drag_Start_Size(E, Element), On_Right_Drag=lambda E: self.Drag_Size(E, Element), On_Right_Release=lambda E: self.Drag_Release(E, Element))
            Element.Bind(On_Double_Click=lambda E: self.Show_Current(Element))
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Drag_Release(self, E, Element):
        try:
            self.Dragging = False
            self.Current = False
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Drag_Start(self, E, Element):
        try:
            if not Element.Lock and not self.Dragging:
                self._Coord = {'X': 0, 'Y': 0}
                self._Coord['X'] = E.x_root
                self._Coord['Y'] = E.y_root
                self.Element_Fixture = Element.Config_Get('Left', 'Top')
                if self.Grid_Lock:
                    Temp_Root = getattr(self.Design, self.Design.Display_ID)
                    Window_Size = Temp_Root.Size()
                    self.Grid_Width = Window_Size[0]/100
                    self.Grid_Height = Window_Size[1]/100
                self.Dragging = True
                self.Current = Element._ID
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        self.Global['Loading'].Hide()
            
    def Drag(self, E, Element):
        try:
            if not Element.Lock and self.Dragging and self.Current==Element._ID and 'Left' in self.Element_Fixture and 'Top' in self.Element_Fixture:
                Difference_Left = (E.x_root - self._Coord['X'])
                Difference_Top = (E.y_root - self._Coord['Y'])
                New_Left = self.Element_Fixture['Left']+Difference_Left
                New_Top = self.Element_Fixture['Top']+Difference_Top
                if self.Grid_Lock:
                    New_Left = round(New_Left / self.Grid_Width)*self.Grid_Width
                    New_Top = round(New_Top / self.Grid_Height)*self.Grid_Height
                Element.Position(Left=New_Left, Top=New_Top)
                if Element._Type=='Frame' or Element._Type=='Canvas' or Element._Type=='Scroll' or Element._Type=='Popup':
                    Element_Type = 'Frame'
                elif 'Canvas_' in Element._Type:
                    Element_Type = 'Item'
                else:
                    Element_Type = 'Widget'
                Element_Data = self.Design.Database.Get(f"SELECT * FROM `{Element_Type}` WHERE `ID`='{Element._ID}'", Keys=True)
                Element_Data = Element_Data[0]
                if Element_Data['Alignment']=='Percentage':
                    Fixture_Reverse = Element._Main.Locate_Reverse(0, 0, New_Left, New_Top)
                else:
                    Fixture_Reverse = [0, 0, round(New_Left, 3), round(New_Top, 3)]
                self.Design.Database.Post(f"UPDATE `{Element_Type}` SET `Left`={Fixture_Reverse[2]}, `Top`={Fixture_Reverse[3]} WHERE `ID`='{Element._ID}'")
                if self.Design.Configure.Current:
                    self.Design.Configure.Current.Movement_Update()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Drag_Start_Size(self, E, Element):
        try:
            if not Element.Lock and not self.Dragging:
                self._Coord = {'X': 0, 'Y': 0}
                self._Coord['X'] = E.x_root
                self._Coord['Y'] = E.y_root
                self.Element_Fixture = Element.Config_Get('Width', 'Height')
                if len(self.Element_Fixture)==2:
                    if self.Grid_Lock:
                        Temp_Root = getattr(self.Design, self.Design.Display_ID)
                        Window_Size = Temp_Root.Size()
                        self.Grid_Width = Window_Size[0]/100
                        self.Grid_Height = Window_Size[1]/100
                    self.Dragging = True
                    self.Current = Element._ID
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        self.Global['Loading'].Hide()
            
    def Drag_Size(self, E, Element):
        try:
            if not Element.Lock and self.Dragging and self.Current==Element._ID and 'Width' in self.Element_Fixture and 'Height' in self.Element_Fixture:
                Difference_Width = (E.x_root - self._Coord['X'])
                Difference_Height = (E.y_root - self._Coord['Y'])
                New_Width = self.Element_Fixture['Width']+Difference_Width
                New_Height = self.Element_Fixture['Height']+Difference_Height
                if self.Grid_Lock:
                    New_Width = round(New_Width / self.Grid_Width)*self.Grid_Width
                    New_Height = round(New_Height / self.Grid_Height)*self.Grid_Height
                Element.Size(Width=New_Width, Height=New_Height)
                if Element._Type=='Frame' or Element._Type=='Canvas' or Element._Type=='Scroll' or Element._Type=='Popup':
                    Element_Type = 'Frame'
                elif 'Canvas_' in Element._Type:
                    Element_Type = 'Item'
                else:
                    Element_Type = 'Widget'
                Element_Data = self.Design.Database.Get(f"SELECT * FROM `{Element_Type}` WHERE `ID`='{Element._ID}'", Keys=True)
                Element_Data = Element_Data[0]
                if Element_Data['Alignment']=='Percentage':
                    Fixture_Reverse = Element._Main.Locate_Reverse(New_Width, New_Height, 0, 0)
                else:
                    Fixture_Reverse = [round(New_Width, 3), round(New_Height, 3), 0, 0]
                self.Design.Database.Post(f"UPDATE `{Element_Type}` SET `Width`={Fixture_Reverse[0]}, `Height`={Fixture_Reverse[1]} WHERE `ID`='{Element._ID}'")
                if self.Design.Configure.Current:
                    self.Design.Configure.Current.Movement_Update()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Show_Current(self, Element, Loading=True):
        try:
            if Loading and not self.Dragging:
                self.Hide_Delete()
                self.Global['Loading'].Show()
                self.Global['GUI'].After(10, lambda: self.Show_Current(Element, Loading=False))
                self.Dragging = True
            else:
                self.Design.Configure.Hide_All()
                self.Design.Configure.Reset_All()
                ID_Tree = getattr(self, Element._ID)
                Parent = self.Tree.Parent(ID_Tree)
                self.Expand_Parent(Parent)
                self.Tree.Select(ID_Tree)
                Configure = getattr(self.Design.Configure, f'Configure_{Element._Type}')
                Configure.Load(Element._ID)
                self.Dragging = False
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Expand_Parent(self, ID):
        try:
            self.Tree.Expand(ID)
            Name = self.Tree.Get(ID)[0]
            if Name and Name!=self.Parent:
                Parent = self.Tree.Parent(ID)
                self.Expand_Parent(Parent)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Default(self, Parent='Root'):
        try:
            self.Parent = Parent
            self.Tree.Remove_All()
            setattr(self, self.Parent, self.Tree.Add(self.Parent, Value=[self.Parent, 'Frame', 0]))
            self.Load()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Reset_All(self):
        try:
            Widget = self.Tree.Get()
            if len(Widget)>0:
                if Widget[0]!=self.Current:
                    self.Current = Widget[0]
                    self.Design.Configure.Hide_All()
                    self.Design.Configure.Reset_All()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Get_Current(self, Loading=True):
        try:
            if Loading:
                self.Hide_Delete()
                self.Global['Loading'].Show()
                self.Global['GUI'].After(10, lambda: self.Get_Current(Loading=False))
            else:
                Widget = self.Tree.Get()
                self.Current = Widget[0]
                if Widget:
                    self.Design.Configure.Hide_All()
                    self.Design.Configure.Reset_All()
                    if Widget[0]!=self.Parent:
                        Configure = getattr(self.Design.Configure, f'Configure_{Widget[1]}')
                        Configure.Load(Widget[0])
                    else:
                        self.Delete_Image.Hide()
                        self.Copy_Image.Hide()
                        if not self.Copy_ID:
                            self.Paste_Image.Hide()
                        self.Global['Loading'].Hide()
                else:
                    self.Global['Loading'].Hide()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Show_Delete(self):
        try:
            self.Delete_Image.Hide()
            self.Copy_Image.Hide()
            self.Paste_Image.Hide()
            self.Delete_Confirm_Image.Show()
            self.Delete_Cancel_Image.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Hide_Delete(self):
        try:
            self.Delete_Confirm_Image.Hide()
            self.Delete_Cancel_Image.Hide()
            self.Delete_Image.Show()
            self.Copy_Image.Show()
            self.Export_Image.Show()
            if self.Copy_ID:
                self.Paste_Image.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Copy(self):
        try:
            ID = self.Tree.Current()
            if ID:
                Element_ID = self.Tree.Get(ID)[0]
                if Element_ID!=self.Parent:
                    self.Copy_ID = ID
                    self.Paste_Image.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Delete(self):
        try:
            ID = self.Tree.Selected()
            for Each in ID:
                self.Delete_All(Each)
            self.Hide_Delete()
            self.Design.Configure.Hide_All()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Delete_All(self, ID=False):
        try:
            if ID:
                if self.Copy_ID==ID:
                    self.Copy_ID = False
                    self.Paste_Image.Hide()
                for Each in self.Tree.Child(ID):
                    self.Delete_All(Each)
                Element_ID = self.Tree.Get(ID)[0]
                if Element_ID!=self.Parent:
                    if len(self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                        Frame_Data = self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    elif len(self.Design.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                        Frame_Data = self.Design.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    else:
                        Frame_Data = self.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    Temp_Root = Frame_Data['Root']
                    Root = Temp_Root
                    while Temp_Root!=self.Parent:
                        Frame_Data = self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Temp_Root}'", Keys=True)
                        if len(Frame_Data)==1:
                            Frame_Data = Frame_Data[0]
                            Temp_Root = Frame_Data['Root']
                            Root = Temp_Root+'.'+Root
                        else:
                            break
                    if os.path.exists(f"{self.Design.Project_Path}/Data/File/{Element_ID}"):
                        os.remove(f"{self.Design.Project_Path}/Data/File/{Element_ID}")
                    Root = self.Global['Custom'].Get_Attr_Class(self.Design, Root)
                    Element = getattr(Root, Element_ID)
                    Element.Delete()
                    self.Tree.Remove(ID)
                    self.Design.Database.Post(f"DELETE FROM `Frame` WHERE `ID`='{Element_ID}'")
                    self.Design.Database.Post(f"DELETE FROM `Item` WHERE `ID`='{Element_ID}'")
                    self.Design.Database.Post(f"DELETE FROM `Widget` WHERE `ID`='{Element_ID}'")
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
    
    def Export(self):
        try:
            if not os.path.exists(f"{self.Design.Project_Path}/Structure"):
                os.mkdir(f"{self.Design.Project_Path}/Structure")
            self.Tree.Export(Path=f"{self.Design.Project_Path}/Structure/{self.Parent}.txt")
            self.Global['Message'].Show('Success', 'Structure Exported To Project Folder')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Paste(self):
        try:
            Root_ID = self.Tree.Current()
            if Root_ID:
                Root_Type = self.Tree.Get(Root_ID)[1]
                if Root_Type=='Frame' or Root_Type=='Canvas' or Root_Type=='Scroll':
                    Error = False
                    Parent = Root_ID
                    while self.Tree.Get(Parent)[0]!=self.Parent:
                        if Parent==self.Copy_ID:
                            Error = True
                            break
                        Parent = self.Tree.Parent(Parent)
                    if not Error:
                        self.Paste_All(self.Copy_ID, Root_ID)
                    else:
                        self.Global['Message'].Show('Error', 'Copy Not Possible Into Same Widget')
                else:
                    self.Global['Message'].Show('Error', 'Copy To Selected Widget Not Posiible')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Paste_All(self, ID=False, Root=False):
        try:
            if ID and Root:
                Root_ID = self.Tree.Get(Root)[0]
                Element_ID = self.Tree.Get(ID)[0]
                if len(self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                    Type = 'Frame'
                elif len(self.Design.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                    Type = 'Item'
                else:
                    Type = 'Widget'
                self.Design.Database.Post(f"DELETE FROM `{Type}_Copy`")
                self.Design.Database.Post(f"INSERT INTO `{Type}_Copy` SELECT * FROM `{Type}` WHERE `ID`='{Element_ID}'")
                Frame_Data = self.Design.Database.Get(f"SELECT * FROM `{Type}_Copy` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                Number = 0
                Exist = self.Design.Database.Get(f"SELECT * FROM `{Type}` WHERE (`Name`='{Frame_Data['Name']}' AND `Root`='{Root_ID}')")
                while len(Exist)>0:
                    Number += 1
                    Exist = self.Design.Database.Get(f"SELECT * FROM `{Type}` WHERE (`Name`='{Frame_Data['Name']}{Number}' AND `Root`='{Root_ID}')")
                if Number>0:
                    Name = f"{Frame_Data['Name']}{Number}"
                else:
                    Name = Frame_Data['Name']
                Random_Letter = ''.join(random.choices(string.ascii_letters, k=10))
                New_ID = Random_Letter+self.Global['Custom'].MD5(Root_ID+Name+str(time.time()*1000000))
                self.Design.Database.Post(f"UPDATE `{Type}_Copy` SET `ID`='{New_ID}',`Name`='{Name}',`Root`='{Root_ID}' WHERE `ID`='{Element_ID}'")
                self.Design.Database.Post(f"INSERT INTO `{Type}` SELECT * FROM `{Type}_Copy` WHERE `ID`='{New_ID}'")
                self.Design.Database.Post(f"DELETE FROM `{Type}_Copy` WHERE `ID`='{New_ID}'")
                if os.path.exists(f"{self.Design.Project_Path}/Data/File/{Element_ID}"):
                    shutil.copy(f"{self.Design.Project_Path}/Data/File/{Element_ID}", f"{self.Design.Project_Path}/Data/File/{New_ID}")
                if Type=='Frame':
                    Root_Level = self.Tree.Get(Root)[2]
                    self.Design.Database.Post(f"UPDATE `{Type}` SET `Level`='{Root_Level+1}' WHERE `ID`='{New_ID}'")
                Widget = self.Design.Database.Get(f"SELECT * FROM `{Type}` WHERE `ID`='{New_ID}'", Keys=True)[0]
                Stock = getattr(self.Design.Stock, f"Stock_{Widget['Type']}")
                Stock.Create(Widget['ID'])
                Level = 0
                if Type=='Frame':
                    Level = Widget['Level']
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=Widget['Name'], Parent=Root, Value=[Widget['ID'], Widget['Type'], Level], Path=Image))
                for Each in self.Tree.Child(ID):
                    self.Paste_All(Each, getattr(self, Widget['ID']))
        except Exception as E:
            traceback.print_exc()
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Grid(self):
        try:
            if self.Grid_Lock:
                self.Grid_Lock = False
                self.Grid_Image.Config(Path=self.Global['Image']('Grid_Unlock'))
            else:
                self.Grid_Lock = True
                self.Grid_Image.Config(Path=self.Global['Image']('Grid_Lock'))
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


################################################################################################################################
#Stock
################################################################################################################################
#Default Libraries
import inspect

#Program
from .N_Stock_Frame import Stock_Frame
from .N_Stock_Canvas import Stock_Canvas
from .N_Stock_Scroll import Stock_Scroll
from .N_Stock_Label import Stock_Label
from .N_Stock_Roubel import Stock_Roubel
from .N_Stock_Label_Lite import Stock_Label_Lite
from .N_Stock_Bar import Stock_Bar
from .N_Stock_Button import Stock_Button
from .N_Stock_Button_Lite import Stock_Button_Lite
from .N_Stock_Compound import Stock_Compound
from .N_Stock_Compound_Lite import Stock_Compound_Lite
from .N_Stock_Image import Stock_Image
from .N_Stock_Image_Lite import Stock_Image_Lite
from .N_Stock_Line import Stock_Line
from .N_Stock_Entry import Stock_Entry
from .N_Stock_List import Stock_List
from .N_Stock_Select import Stock_Select
from .N_Stock_Spinner import Stock_Spinner
from .N_Stock_Scale import Stock_Scale
from .N_Stock_Check import Stock_Check
from .N_Stock_Radio import Stock_Radio
from .N_Stock_Switch import Stock_Switch
from .N_Stock_Text import Stock_Text
from .N_Stock_Tree import Stock_Tree
from .N_Stock_Canvas_Line import Stock_Canvas_Line
from .N_Stock_Canvas_Rectangle import Stock_Canvas_Rectangle
from .N_Stock_Canvas_Pie import Stock_Canvas_Pie
from .N_Stock_Canvas_Arc import Stock_Canvas_Arc
from .N_Stock_Canvas_Circle import Stock_Canvas_Circle
from .N_Stock_Canvas_Oval import Stock_Canvas_Oval
from .N_Stock_Canvas_Image import Stock_Canvas_Image
from .N_Stock_Canvas_Text import Stock_Canvas_Text

class Stock:
    def __init__(self, Global, Design):
        try:
            self.Global = Global
            self.Design = Design
            self.Widget = []
            
            #Line
            Fixture = self.Design.Frame.Locate(0.5, 90, 28.5, 10)
            self.Line = self.Global['Gluonix'].Line(self.Design.Frame)
            self.Line.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Line.Config(Border_Size=0, Display=True)
            self.Line.Config(Resize=False, Move=False)
            self.Line.Create()
            self.Design.Widget.append(self.Line)
            
            #Frame
            Fixture = self.Design.Frame.Locate(29.5, 90, 0, 10)
            self.Frame = self.Global['Gluonix'].Frame(self.Design.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Frame.Config(Resize=False, Move=False)
            self.Frame.Create()
            self.Design.Widget.append(self.Frame)
            
            #Label
            Fixture = self.Frame.Locate(100, 3, 0, 1)
            self.Label = self.Global['Gluonix'].Label(self.Frame)
            self.Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Label.Config(Foreground='#000000', Font_Size=16, Font_Weight='normal', Align='w')
            self.Label.Config(Resize=True, Move=True)
            self.Label.Set(' ELEMENTS')
            self.Label.Create()
            
            #Frame
            Fixture = self.Frame.Locate(100, 95, 0, 5)
            self.Scroll = self.Global['Gluonix'].Scroll(self.Frame)
            self.Scroll.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Scroll.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Scroll.Config(Resize=False, Move=False)
            self.Scroll.Create()
            
            #Objects
            
            self.Top = 3
            self.Left = 2.5
            self.Center = 35
            self.Right = 67.5
            
            # Containers
            
            #Label
            Fixture = self.Scroll.Locate(100, 3, 0, self.Top)
            self.Label_Container = self.Global['Gluonix'].Label(self.Scroll)
            self.Label_Container.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label_Container.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Label_Container.Config(Foreground='#000000', Font_Size=12, Font_Weight='normal', Align='w')
            self.Label_Container.Config(Resize=True, Move=True)
            self.Label_Container.Set(' CONTAINER')
            self.Label_Container.Create()
            
            #Stock Frame
            self.Top += 4
            self.Stock_Frame = Stock_Frame(self.Global, self, self.Left, self.Top)
            
            #Stock Canvas
            self.Stock_Canvas = Stock_Canvas(self.Global, self, self.Center, self.Top)
            
            #Stock Scroll
            self.Stock_Scroll = Stock_Scroll(self.Global, self, self.Right, self.Top)
            
            # Widget
            
            #Label
            self.Top += 7
            Fixture = self.Scroll.Locate(100, 3, 0, self.Top)
            self.Label_Widget = self.Global['Gluonix'].Label(self.Scroll)
            self.Label_Widget.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label_Widget.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Label_Widget.Config(Foreground='#000000', Font_Size=12, Font_Weight='normal', Align='w')
            self.Label_Widget.Config(Resize=True, Move=True)
            self.Label_Widget.Set(' WIDGET')
            self.Label_Widget.Create()
            
            #Stock Line
            self.Top += 4
            self.Stock_Line = Stock_Line(self.Global, self, self.Left, self.Top)
            
            #Stock Label
            self.Stock_Label = Stock_Label(self.Global, self, self.Center, self.Top)
            
            #Stock Button
            self.Stock_Button = Stock_Button(self.Global, self, self.Right, self.Top)
            
            #Stock Image
            self.Top += 6
            self.Stock_Image = Stock_Image(self.Global, self, self.Left, self.Top)
            
            #Stock Compound
            self.Stock_Compound = Stock_Compound(self.Global, self, self.Center, self.Top)
            
            #Stock Text
            self.Stock_Text = Stock_Text(self.Global, self, self.Right, self.Top)
            
            #Stock Entry
            self.Top += 6
            self.Stock_Entry = Stock_Entry(self.Global, self, self.Left, self.Top)
            
            #Stock Spinner
            self.Stock_Spinner = Stock_Spinner(self.Global, self, self.Center, self.Top)
            
            #Stock Select
            self.Stock_Select = Stock_Select(self.Global, self, self.Right, self.Top)
            
            #Stock Bar
            self.Top += 6
            self.Stock_Bar = Stock_Bar(self.Global, self, self.Left, self.Top)
            
            #Stock List
            self.Stock_List = Stock_List(self.Global, self, self.Center, self.Top)
            
            #Stock Scale
            self.Stock_Scale = Stock_Scale(self.Global, self, self.Right, self.Top)
            
            #Stock Check
            self.Top += 6
            self.Stock_Check = Stock_Check(self.Global, self, self.Left, self.Top)
            
            #Stock Radio
            self.Stock_Radio = Stock_Radio(self.Global, self, self.Center, self.Top)
            
            #Stock Switch
            self.Stock_Switch = Stock_Switch(self.Global, self, self.Right, self.Top)
            
            #Stock Tree
            self.Top += 6
            self.Stock_Tree = Stock_Tree(self.Global, self, self.Left, self.Top)
            
            #Stock Roubel
            self.Stock_Roubel = Stock_Roubel(self.Global, self, self.Center, self.Top)
            
            # Lite
            
            #Label
            self.Top += 7
            Fixture = self.Scroll.Locate(100, 3, 0, self.Top)
            self.Label_Lite = self.Global['Gluonix'].Label(self.Scroll)
            self.Label_Lite.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label_Lite.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Label_Lite.Config(Foreground='#000000', Font_Size=12, Font_Weight='normal', Align='w')
            self.Label_Lite.Config(Resize=True, Move=True)
            self.Label_Lite.Set(' LITE WIDGET')
            self.Label_Lite.Create()
            
            #Stock Label Lite
            self.Top += 4
            self.Stock_Label_Lite = Stock_Label_Lite(self.Global, self, self.Left, self.Top)
            
            #Stock Image Lite
            self.Stock_Image_Lite = Stock_Image_Lite(self.Global, self, self.Center, self.Top)
            
            #Stock Compound Lite
            self.Stock_Compound_Lite = Stock_Compound_Lite(self.Global, self, self.Right, self.Top)
            
            #Stock Button Lite
            self.Top += 6
            self.Stock_Button_Lite = Stock_Button_Lite(self.Global, self, self.Left, self.Top)
            
            # Lite
            
            #Label
            self.Top += 7
            Fixture = self.Scroll.Locate(100, 3, 0, self.Top)
            self.Label_Lite = self.Global['Gluonix'].Label(self.Scroll)
            self.Label_Lite.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label_Lite.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Label_Lite.Config(Foreground='#000000', Font_Size=12, Font_Weight='normal', Align='w')
            self.Label_Lite.Config(Resize=True, Move=True)
            self.Label_Lite.Set(' CANVAS ITEM')
            self.Label_Lite.Create()
            
            #Stock Canvas Line
            self.Top += 4
            self.Stock_Canvas_Line = Stock_Canvas_Line(self.Global, self, self.Left, self.Top)
            
            #Stock Canvas Rectangle
            self.Stock_Canvas_Rectangle = Stock_Canvas_Rectangle(self.Global, self, self.Center, self.Top)
            
            #Stock Canvas Circle
            self.Stock_Canvas_Circle = Stock_Canvas_Circle(self.Global, self, self.Right, self.Top)
            
            #Stock Canvas Oval
            self.Top += 6
            self.Stock_Canvas_Oval = Stock_Canvas_Oval(self.Global, self, self.Left, self.Top)
            
            #Stock Canvas Image
            self.Stock_Canvas_Image = Stock_Canvas_Image(self.Global, self, self.Center, self.Top)
            
            #Stock Canvas Text
            self.Stock_Canvas_Text = Stock_Canvas_Text(self.Global, self, self.Right, self.Top)
            
            #Stock Canvas Arc
            self.Top += 6
            self.Stock_Canvas_Arc = Stock_Canvas_Arc(self.Global, self, self.Left, self.Top)
            
            #Stock Canvas Pie
            self.Stock_Canvas_Pie = Stock_Canvas_Pie(self.Global, self, self.Center, self.Top)
            
            #Update Scroll
            #self.Scroll.Update(self.Stock_Button_Lite.Label)
                   
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


# IMPORT LIBRARIES
import os
import math
from io import BytesIO
from requests import get as requests_get
from PIL import Image as PIL_Image, ImageTk as PIL_ImageTk
from .N_Custom import Event_Bind_Canvas
        
class Canvas_Image:
    def __init__(self, Main):
        self._Canvas = Main
        self._Config = ['Width', 'Height', 'Left', 'Top', 'Anchor', 'Url', 'Array', 'Pil', 'Photo', 'Resize', 'Rotate', 'Path', 'Path_Initial', 'Transparent']
        self._Display = True
        self._Resize = True
        self._Name = False
        self._Last_Name = False
        self._Type = 'Canvas_Image'
        self._Anchor = 'nw'
        self._Image = False
        self._Image_Garbage = False
        self._Path = False
        self._Path_Memory = False
        self._Path_Initial = False
        self._Url = False
        self._Array = False
        self._Pil = False
        self._Photo = False
        self._Transparent = True
        self._Rotate = 0
        self._Angle = 0
        self._Width, self._Height, self._Width_Old, self._Height_Old, self._Left, self._Top = 0, 0, 0, 0, 0, 0
        self._Widget = self._Canvas._Frame.create_image(0, 0, anchor=self._Anchor, image=None)
        self._Canvas._Widget.append(self)
        self._Resizable = self._Canvas._Resizable
        self._On_Show = False
        self._On_Hide = False

    def Copy(self, Name=False, Main=False):
        try:
            if not Main:
                Main = self._Canvas
            Instance = type(self)(Main)
            for Key in self._Config:
                setattr(Instance, "_"+Key, getattr(self, "_"+Key))
            setattr(Instance, "_Name", Name)
            Instance.Create()
            return Instance
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Copy -> {E}")

    def Hide(self):
        try:
            self._Canvas._Frame.itemconfigure(self._Widget, state='hidden')
            self._Display = False
            if self._On_Hide:
                self._On_Hide()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type}-> Hide -> {E}")
            
    def Show(self):
        try:
            self._Display = True
            if self._Resizable:
                self.Resize()
            else:
                self.Display()
            if self._On_Show:
                self._On_Show()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Show -> {E}")

    def Display(self):
        try:
            self._Canvas._Frame.itemconfigure(self._Widget, state='normal')
            self._Canvas._Frame.tag_raise(self._Widget)
            self._Display = True
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Display -> {E}")
    
    def Delete(self):
        try:
            self._Canvas._Widget.remove(self)
            self._Canvas._Frame.delete(self._Widget)
            if self:
                del self
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Delete -> {E}")
        
    def Set(self, Path):
        try:
            self._Path = Path
            self._Path_Memory = self._Path
            self.Open()
            self.Load()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Set -> {E}")
            
    def Initial(self):
        try:
            if self._Path_Initial:
                Load_Setup = [self._Array, self._Url, self._Pil, self._Photo]
                self._Array, self._Url, self._Pil, self._Photo = False, False, False, False
                self.Set(self._Path_Initial)
                self._Array, self._Url, self._Pil, self._Photo = Load_Setup
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Initial -> {E}")
            
    def Refresh(self):
        try:
            self.Open()
            self.Load()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Refresh -> {E}")
        
    def Bind(self, **Input):
        try:
            if 'On_Show' in Input:
                self._On_Show = Input['On_Show']
            if 'On_Hide' in Input:
                self._On_Hide = Input['On_Hide']
            Event_Bind_Canvas(self._Canvas._Frame, self._Widget, **Input)
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Bind -> {E}")
            
    def Config_Get(self, *Input):
        try:
            Return = {}
            for Each in self._Config:
                if Each in Input:
                    Return[Each] = getattr(self, "_"+Each)
            return Return
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Config_Get -> {E}")
                
    def Config(self, **Input):
        try:
            Run = False
            for Each in self._Config:
                if Each in Input:
                    Value = Input[Each]
                    setattr(self, "_"+Each, Value)
                    setattr(self, "_"+Each+"_Current", Value)
                    Run = True
            if Run:
                self.Relocate()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Config -> {E}")
        
    def Position(self, Left=None, Top=None):
        try:
            if Left is not None:
                self._Left = Left
            if Top is not None:
                self._Top = Top
            if Left is not None or Top is not None:
                self.Relocate()
            Box = self._Canvas._Frame.bbox(self._Widget)
            X1, Y1, X2, Y2 = Box
            return [X1, Y1]
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Position -> {E}")
            
    def Size(self, Width=False, Height=False):
        try:
            if Width:
                self._Width = Width
            if Height:
                self._Height = Height
            if Width or Height:
                self.Relocate()
            Box = self._Canvas._Frame.bbox(self._Widget)
            X1, Y1, X2, Y2 = Box
            Width = X2 - X1
            Height = Y2 - Y1
            return [Width, Height]
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Size -> {E}")
        
    def Open(self):
        try:
            if self._Image:
                self._Image.close()
            if self._Url:
                if self._Path:
                    Image_Data = requests_get(self._Path)
                    self._Image = PIL_Image.open(BytesIO(Image_Data.content))
            elif self._Array:
                if self._Path is not None:
                    self._Image = PIL_Image.fromarray(self._Path)
            elif self._Pil:
                if self._Path:
                    self._Image = self._Path.copy()
            elif self._Photo:
                if self._Path:
                    self._Image = self._Path
            else:
                if self._Path and os.path.exists(self._Path):
                    self._Image = PIL_Image.open(self._Path)
                    if not self._Path_Initial:
                        self._Path_Initial = self._Path
            if self._Image and not self._Photo:
                self._Image_Width, self._Image_Height = self._Image.size
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Open -> {E}")
        
    def Convert(self):
        try:
            Temp_Image = self._Image.rotate(self._Rotate+self._Angle, PIL_Image.NEAREST, expand=0)
            if self._Transparent:
                Temp_Image_Convert = Temp_Image.convert('RGBA')
            else:
                Temp_Image_Convert = Temp_Image.convert("HSV")
            Width, Height = Temp_Image.size
            if self._Width_Current>0 and self._Height_Current>0:
                Temp_Image = Temp_Image.resize((int(self._Width_Current), int(self._Height_Current)), PIL_Image.NEAREST)
            elif self._Width_Current and self._Height_Current<1:
                Ratio = Height/Width
                Temp_Image = Temp_Image.resize((int(self._Width_Current), int(self._Width_Current*Ratio)), PIL_Image.NEAREST)
            elif self._Width_Current<1 and self._Height_Current:
                Ratio = Width/Height
                Temp_Image = Temp_Image.resize((int(self._Height_Current*Ratio), int(self._Height_Current)), PIL_Image.NEAREST)
            else:
                Temp_Image = Temp_Image.copy()
            self._Width_Old, self._Height_Old = self._Width_Current, self._Height_Current
            Temp_Image_TK = PIL_ImageTk.PhotoImage(Temp_Image_Convert)
            return Temp_Image_TK
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Convert -> {E}")
        
    def Load(self):
        try:
            if self._Image:
                Image = self._Image  if self._Photo else self.Convert()
                self._Image_Garbage = Image
                self._Canvas._Frame.itemconfig(self._Widget, image=Image)
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Load -> {E}")
        
    def Create(self):
        try:
            self._Canvas._Frame.itemconfig(self._Widget, anchor=self._Anchor)
            self._Canvas._Frame.coords(self._Widget, self._X_Current, self._Y_Current)
            if not self._Image:
                self.Open()
            self.Load()
            if self._Name!=self._Last_Name:
                if self._Last_Name:
                    if self._Last_Name in self._Canvas.__dict__:
                        del self._Canvas.__dict__[self._Last_Name]
                if self._Name:
                    self._Canvas.__dict__[self._Name] = self
                self._Last_Name = self._Name
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Create -> {E}")
            
    def Rotate(self, Value=0):
        try:
            self._Angle+=Value
            self.Load()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Rotate -> {E}")

    def Adjustment(self):
        try:
            self._Width_Ratio = self._Canvas._Width_Current / self._Canvas._Width
            self._Height_Ratio = self._Canvas._Height_Current / self._Canvas._Height
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Adjustment -> {E}")
            
    def Relocate(self, Direct=False):
        try:
            if self._Resize:
                self.Adjustment()
                self._X_Current = self._Left * self._Width_Ratio
                self._Y_Current = self._Top * self._Height_Ratio
                self._Width_Current = self._Width * self._Width_Ratio
                self._Height_Current = self._Height * self._Height_Ratio
            self.Create()
            if self._Display:
                self.Display()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Relocate -> {E}")
            
    def Resize(self):
        try:
            self.Relocate()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Resize -> {E}")
            


################################################################################################################################
#Configure
################################################################################################################################
#Default Libraries
import inspect

#Program
from .N_Configure_Frame import Configure_Frame
from .N_Configure_Canvas import Configure_Canvas
from .N_Configure_Scroll import Configure_Scroll
from .N_Configure_Label import Configure_Label
from .N_Configure_Roubel import Configure_Roubel
from .N_Configure_Label_Lite import Configure_Label_Lite
from .N_Configure_Bar import Configure_Bar
from .N_Configure_Button import Configure_Button
from .N_Configure_Button_Lite import Configure_Button_Lite
from .N_Configure_Compound import Configure_Compound
from .N_Configure_Compound_Lite import Configure_Compound_Lite
from .N_Configure_Image import Configure_Image
from .N_Configure_Image_Lite import Configure_Image_Lite
from .N_Configure_Line import Configure_Line
from .N_Configure_Entry import Configure_Entry
from .N_Configure_List import Configure_List
from .N_Configure_Select import Configure_Select
from .N_Configure_Spinner import Configure_Spinner
from .N_Configure_Scale import Configure_Scale
from .N_Configure_Check import Configure_Check
from .N_Configure_Radio import Configure_Radio
from .N_Configure_Switch import Configure_Switch
from .N_Configure_Text import Configure_Text
from .N_Configure_Tree import Configure_Tree
from .N_Configure_Canvas_Line import Configure_Canvas_Line
from .N_Configure_Canvas_Rectangle import Configure_Canvas_Rectangle
from .N_Configure_Canvas_Pie import Configure_Canvas_Pie
from .N_Configure_Canvas_Arc import Configure_Canvas_Arc
from .N_Configure_Canvas_Circle import Configure_Canvas_Circle
from .N_Configure_Canvas_Oval import Configure_Canvas_Oval
from .N_Configure_Canvas_Image import Configure_Canvas_Image
from .N_Configure_Canvas_Text import Configure_Canvas_Text

class Configure:
    def __init__(self, Global, Design):
        try:
            self.Global = Global
            self.Design = Design
            self.Widget= []
            self.Current = False
            
            #Frame
            Fixture = self.Design.Frame.Locate(40, 90, 60, 10)
            self.Frame = self.Global['Gluonix'].Frame(self.Design.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Frame.Config(Resize=False, Move=False)
            self.Frame.Create()
            self.Design.Widget.append(self.Frame)
            
            #Label
            Fixture = self.Frame.Locate(100, 3, 0, 1)
            self.Label = self.Global['Gluonix'].Label(self.Frame)
            self.Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Label.Config(Foreground='#000000', Font_Size=16, Font_Weight='normal', Align='w')
            self.Label.Config(Resize=False, Move=False)
            self.Label.Set(' CONFIGURE')
            self.Label.Create()
            
            #Configure Frame
            self.Configure_Frame = Configure_Frame(self.Global, self)
            
            #Configure Canvas
            self.Configure_Canvas = Configure_Canvas(self.Global, self)
            
            #Configure Scroll
            self.Configure_Scroll = Configure_Scroll(self.Global, self)
            
            #Configure Label
            self.Configure_Label = Configure_Label(self.Global, self)
            
            #Configure Roubel
            self.Configure_Roubel = Configure_Roubel(self.Global, self)
            
            #Configure Bar
            self.Configure_Bar = Configure_Bar(self.Global, self)
            
            #Configure Button
            self.Configure_Button = Configure_Button(self.Global, self)
            
            #Configure Compound
            self.Configure_Compound = Configure_Compound(self.Global, self)
            
            #Configure Image
            self.Configure_Image = Configure_Image(self.Global, self)
            
            #Configure Line
            self.Configure_Line = Configure_Line(self.Global, self)
            
            #Configure Entry
            self.Configure_Entry = Configure_Entry(self.Global, self)
            
            #Configure List
            self.Configure_List = Configure_List(self.Global, self)
            
            #Configure Select
            self.Configure_Select = Configure_Select(self.Global, self)
            
            #Configure Spinner
            self.Configure_Spinner = Configure_Spinner(self.Global, self)
            
            #Configure Scale
            self.Configure_Scale = Configure_Scale(self.Global, self)
            
            #Configure Check
            self.Configure_Check = Configure_Check(self.Global, self)
            
            #Configure Radio
            self.Configure_Radio = Configure_Radio(self.Global, self)
            
            #Configure Switch
            self.Configure_Switch = Configure_Switch(self.Global, self)
            
            #Configure Text
            self.Configure_Text = Configure_Text(self.Global, self)
            
            #Configure Text
            self.Configure_Tree = Configure_Tree(self.Global, self)
            
            #Configure Text
            self.Configure_Label_Lite = Configure_Label_Lite(self.Global, self)
            
            #Configure Text
            self.Configure_Compound_Lite = Configure_Compound_Lite(self.Global, self)
            
            #Configure Text
            self.Configure_Image_Lite = Configure_Image_Lite(self.Global, self)
            
            #Configure Text
            self.Configure_Button_Lite = Configure_Button_Lite(self.Global, self)
            
            #Configure Canvas Line
            self.Configure_Canvas_Line = Configure_Canvas_Line(self.Global, self)
            
            #Configure Canvas Rectangle
            self.Configure_Canvas_Rectangle = Configure_Canvas_Rectangle(self.Global, self)
            
            #Configure Canvas Arc
            self.Configure_Canvas_Pie = Configure_Canvas_Pie(self.Global, self)
            
            #Configure Canvas Arc
            self.Configure_Canvas_Arc = Configure_Canvas_Arc(self.Global, self)
            
            #Configure Canvas Circle
            self.Configure_Canvas_Circle = Configure_Canvas_Circle(self.Global, self)
            
            #Configure Canvas Oval
            self.Configure_Canvas_Oval = Configure_Canvas_Oval(self.Global, self)
            
            #Configure Canvas Image
            self.Configure_Canvas_Image = Configure_Canvas_Image(self.Global, self)
            
            #Configure Canvas Text
            self.Configure_Canvas_Text = Configure_Canvas_Text(self.Global, self)
            
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Hide_All(self):
        try:
            self.Current = False
            for Each in self.Widget:
                Each.Frame.Top()
                Each.Frame.Hide()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Reset_All(self):
        try:
            for Each in self.Widget:
                Each.ID = False
                Each.Root = False
                Each.Root_ID = False
                Each.Element = False
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


from setuptools import setup, find_packages

setup(
    name="GluonixDesigner",
    version="3.4",
    description="A GUI Designer with drag-and-drop features.",
    author="Nucleon Automation",
    author_email="jagroop@nucleonautomation.com",
    packages=find_packages(),
    install_requires=[
        "pillow>=6.0.0",
        "requests>=2.22.0"
    ],
    entry_points={
        "console_scripts": [
            "GluonixDesigner = GluonixDesigner.Designer:main",
            "Gluonix = GluonixDesigner.Designer:main"
        ]
    },
    include_package_data=True,
    package_data={
        "GluonixDesigner": ["Data/**/*"],
    },
)

'''
python -m build --sdist
twine upload dist/*
pip install dist/GluonixDesigner-2.5.tar.gz
GluonixDesigner
'''